; Climber 5
; Copyright 2003 Dennis Debro
;
; dasm source.s -f3 -osource.bin to compile
;
; Tested with Z26, Atari7800, Atari2600 (4-switch) via Cuttle Cart
;
; Stella doesn't show the ball in the upper left hand corner so the game is
; unplayable in this emulator :(
;
; This game displays 200 scan lines for the main kernel. The vertical blank
; and overscan time was adjusted to compensate for this.
;
; This is basically the version that was shown a PhillyClassic4. There have
; been minor changes since then:
;   o Climber 5 font
;   o game colors cycle after level 5
;   o added a fair pixel distance for the girders so the climber can squeeze
;     between them
;   o PAL timings changed (thanks to Fabrizio Zavagli (rasty) for testing)
;   o optimizations (saved a couple of ROM bytes)
;
; I have commented the code to help others understand my thought process. Skip
; the areas where you think I over commented :)
;
; I tried to stick to the programming standards set by Atari in 1987. You can
; read the PDF document from
; http://www.neonghost.com/the-dig/dox/2600_Standards.pdf.
;
; This was originally posted to the Stella list on Apr.16, 2003. I received
; many suggestions from Andrew Davie and Thomas Jentzsch (commented). These
; suggestions have helped me reduce the ROM by 190 bytes!! The original posting
; used 3558 bytes of ROM.
;
; This is to be a close rendition of the 8-bit version published by COMPUTE!
; I present this in public domain for educational purposes. Please DO NOT make
; any cartridges for sell using this code. It's public domain but it is owned
; by me!
;
;Instructions
;------------
;
;PLAY BALL!
;----------
;What a beautiful day for a baseball game. Sure your friends laugh at you
;because you're the ball boy but you made the team. The bad thing about being
;the ball boy is that the games are played with only one baseball. So if a
;batter hits a homerun the game waits for you to bring the ball back to the
;park.
;
;You have an important job as ball boy. That's why you have the number 5 on
;your back to show that you're an important part of the team.
;
;The opposing team comes up to bat. The pitch is thrown and "WHACK!" it's a
;homerun. The ball flies out of the ball park and lands on a partially
;constructed building across the street. Sure it's dangerous to climb the
;construction site and avoiding the moving girders but that's you're job as
;ball boy. The team is counting on you.
;
;PLAYING THE GAME
;----------------
;You control the ball boy (Climber 5) with the joystick plugged into port 0.
;LEFT = Climber 5 moves left
;RIGHT = Climber 5 moves right
;UP = Climber 5 moves up
;DOWN = Climber 5 moves down
;SELECT = Select game variation
;RESET = Restart the game for the selected game variation
;B&W (7800 Pause)= Pause
;
;Use the RIGHT DIFFICULTY switch to adjust colors for NTSC or PAL televisions.
;
;THE FIRE BUTTON IS NOT USED FOR THIS GAME.
;
;The baseball is located on the top rafter. Use the ladders provided to ascend
;to the top to retrieve the ball. When the climber touches the ball a success
;tone will play and the level of difficulty increases.
;
;GAME VARIATIONS
;---------------
;
;TRAINING
;o This mode will periodically add a new girder each round. Once all seven
;girders are in place the game will progress as normal.
;
;NORMAL
;o A girder is placed on each platform. The climber must dodge these girders to
;reach the baseball safetly.
;
;NOTES
;-----
;When the game is paused for a period of time (~10 minutes) the screen will go
;black. This is put in place to protect your television from burn out. Move
;the joystick to resume screen output.
;
;ACKNOWLEDGMENTS
;---------------
;Climber 5 is based on an Atari 8-bit public domain game of the same name. The
;original game was written by James Rogers and appeared in COMPUTE! magazine
;(Aug. 1987 - Vol 9, No. 8, Iss. 87).
;
;This game uses snipplets from Peiro Cavina's PCMCD demo which later became
;Oystron. Peiro gave me permission to use his kernel. Thanks Peiro!!
;
;The PAL timings were test for me by Fabrizio Zavagli (rasty). He sent a number
;of ROMs for him to test for me. He really went out of his way.
;
;This game also includes help from other StellaList members. If it had not been
;for them sharing code and showing examples there would have been no way for me
;to finish this. LONG LIVE THE STELLA LIST AND LONG LIVE THE ATARI VCS!!!
;
;Thanks for all of your help.
;
   processor 6502
   include ..\vcs.h
   

;============================================================================
; C O N S T A N T S
;============================================================================
NUMGRP               =  $07         ; number of graphics/platform levels
GRPHEIGHT            =  $14         ; height of each girder
PLAYER_HEIGHT        =  $12         ; height of the climber
MAXSELECTLEVEL       =  $01         ; maximum possible selected level

XMIN                 =  $02         ; minimum x value for players
XMAX                 =  $95         ; maximum x value for players

YMIN                 =  $18         ; minimum y value for players 
YMAX                 =  $A8         ; maximum y value for players

SELECT_DELAY         =  $3F         ; delay count for select button (updates ~
                                    ; every second)

GAME_SOUND_VOLUME    =  $A6         ; volume for game sounds

STARTING_LIVES       =  $02         ; number of lives to start - DO NOT CHANGE
                                    ; this is also used to make multiple copies
                                    ; of GRP0 for the player indicator display

CLIMBER_X_START      =  XMAX-4      ; horizontal postion to start the climber
CLIMBER_Y_START      =  $A8         ; vertical position to start the climber

GIRDER_GRAPHIC       =  %11000000   ; graphic data for the girders (STATIC)

MAX_LEVEL            =  $04         ; max number for speed look up

LEVEL_MAX_OUT        =  $99         ; highest level player can achieve

STARTING_LEVEL       =  $01         ; level to start the game

FAIR_PIXEL_DELTA     =  $1B

RESERVE_LIVES_HEIGHT =  $08         ; the height of the lives indicators

BW_MASK              =  %00001000   ; mask for SWCHB to get the B&W value
;----------------------------------------------------------------------------
; ladder graphic values
;
ODD_LADDER_PF1       =  %11000000
ODD_LADDER_PF2       =  %00001100

EVEN_LADDER_PF1      =  %00000110
EVEN_LADDER_PF2      =  %10000000

;----------------------------------------------------------------------------
; NTSC frame constants
;
NTSC_VBLANK_TIME     =  $26         ; ((33 scanlines * 76 cycles) - 14) / 64
NTSC_OVERSCAN_TIME   =  $1F         ; ((26 scanlines * 76 cycles) - 14) / 64
NTSC_SPEED_CONSTANT  =  $1E

;----------------------------------------------------------------------------
; PAL frame constants
;
PAL_VBLANK_TIME      =  $45         ; ((59 scanlines * 76 cycles) - 14) / 64
PAL_OVERSCAN_TIME    =  $3B         ; ((50 scanlines * 76 cycles) - 14) / 64
PAL_SPEED_CONSTANT   =  $24

;----------------------------------------------------------------------------
; game state options
;
GAMERUNNING          =  #%00000000
TITLESCREEN          =  #%10000000
GAMEPAUSED           =  #%01000000
GAME_OVER_STATE      =  #%00000010
START_NEW_GAME       =  #%00000001
TRAINING_MODE        =  #%10000000

;----------------------------------------------------------------------------
; game speed
;

;
; originally was...
;NTSC_LEVEL_1_SPEED   =  $04
;NTSC_LEVEL_2_SPEED   =  $03
;NTSC_LEVEL_3_SPEED   =  $02
;NTSC_LEVEL_4_SPEED   =  $01
;
;PAL_LEVEL_1_SPEED    =  $03         ; 4/60 = x/50 = 15x=50 x = 3.3333~3
;PAL_LEVEL_2_SPEED    =  $03         ; 3/60 = x/50 = 2x=5 x = 2.5 ~3
;PAL_LEVEL_3_SPEED    =  $02         ; 2/60 = x/50 = 3x=5 x = 1.666~2
;PAL_LEVEL_4_SPEED    =  $01         ; 1/60 = x/50 = 6x=5 x =0.83333~1

;
; suggested by Thomas Jentzsch
;
NTSC_LEVEL_1_SPEED   =  $33-1;30*4-1
NTSC_LEVEL_2_SPEED   =  $40-1;30*3-1
NTSC_LEVEL_3_SPEED   =  $55-1;30*2-1
NTSC_LEVEL_4_SPEED   =  $80-1;30*1-1

PAL_LEVEL_1_SPEED    =  $3D-1;36*4-1
PAL_LEVEL_2_SPEED    =  $4C-1;36*3-1
PAL_LEVEL_3_SPEED    =  $66-1;36*2-1
PAL_LEVEL_4_SPEED    =  $9A-1;36*1-1



;----------------------------------------------------------------------------
; color constants
;
MAX_COLOR_ENTRY      =  MAX_LEVEL

BLACK                =  $00
LADDER_COLOR         =  $99

NTSC_LTBLUE          =  $78
NTSC_BALL_COLOR      =  $57
NTSC_GIRDER_COLOR    =  $5C
NTSC_BLUE            =  $86
NTSC_PEACH           =  $3C
NTSC_GREEN           =  $DF
NTSC_RED             =  $40
NTSC_PINK            =  $3F
NTSC_SKY_BLUE        =  $9C
NTSC_WOOD_BROWN      =  $F5
NTSC_BROWN           =  $F9
NTSC_WHITE           =  $0F
NTSC_ATARI_AGE_BLUE  =  $84

PAL_BLACK            =  BLACK
PAL_LTBLUE           =  $BA
PAL_BALL_COLOR       =  $AA
PAL_GIRDER_COLOR     =  $8C
PAL_BLUE             =  $B6
PAL_PEACH            =  $4C
PAL_GREEN            =  $5A
PAL_RED              =  $64
PAL_PINK             =  $6E
PAL_SKY_BLUE         =  $BC
PAL_WOOD_BROWN       =  $20
PAL_WHITE            =  NTSC_WHITE
PAL_BROWN            =  $24
PAL_ATARI_AGE_BLUE   =  $D4

;----------------------------------------------------------------------------
; joystick mask values
;
DOWN_MOTION_MASK     =  %00100000
UP_MOTION_MASK       =  %00010000

;----------------------------------------------------------------------------
; font heights
;
AA_FONT_HEIGHT       =  $0F
PRESENTS_FONT_HEIGHT =  $08
CLIMBER_FONT_HEIGHT  =  $20
NUMBER_5_HEIGHT      =  $06
COPYRIGHT_HEIGHT     =  $08
TEXT_HEIGHT          =  $05
GAME_TEXT_HEIGHT     =  $05

;----------------------------------------------------------------------------
; fine horizontal movement constants
;
HMOVE_L7             =  $70
HMOVE_L6             =  $60
HMOVE_L5             =  $50
HMOVE_L4             =  $40
HMOVE_L3             =  $30
HMOVE_L2             =  $20
HMOVE_L1             =  $10
HMOVE_0              =  $00
HMOVE_R1             =  $F0
HMOVE_R2             =  $E0
HMOVE_R3             =  $D0
HMOVE_R4             =  $C0
HMOVE_R5             =  $B0
HMOVE_R6             =  $A0
HMOVE_R7             =  $90
HMOVE_R8             =  $80

;============================================================================
; Z P - V A R I A B L E S
;============================================================================
   SEG.U variables
   org $80
   
; There is only 128 bytes of RAM available to the programmer. Locations
; $80 - $FF are available for RAM use.
;

random               ds 1           ; holds a random number
gameSelection        ds 1           ; game variation selected (D7 high for
                                    ; training)
gameState            ds 1           ; current game state (see the game
                                    ; state options constants for valid
                                    ; values)
lives                ds 1           ; number of lives available to the player
backgroundColor      ds 1           ; these colors are stored in RAM so they
platformColor        ds 1           ; can be manipulated for NTSC or PAL
verPosP0             ds 1           ; vertical position of climber
horPosP0             ds 1           ; horizontal position of climber
attractMode          ds 1           ; this timer is updated each 255th frame
                                    ; once D7 goes high ($80) the screen
                                    ; display is suppressed
horPosP1             ds NUMGRP      ; array of horizontal positions for the
                                    ; girders
girderGraphics       ds NUMGRP      ; stored in RAM so the training option
                                    ; can show random girders
startCount           ds NUMGRP      ; count down for girder movement
playerMotion         ds 1           ; speed of girders (all the same speed)
climberMotion        ds 1           ; speed of climber
levelBCD             ds 1           ; level number in BCD
playerReflect        ds 1           ; holds the reflect value for the climber
frameCount           ds 1           ; updated each frame
selectDebounce       ds 1           ; debounce flag for the select switch
temp                 ds 1           ; temporary variable for various things
fireButtonDebounce   ds 1           ; debounce flag for the fire button
                                    ; (used for starting a new game)
playerState          ds 1           ; collision state of the climber
soundCounter         ds 1           ; used to play the success and death
soundIndex           ds 1           ; sounds
gameClock            ds 1           ; update periodically based on climber
                                    ; speed
playerPointer        ds 2           ; pointer to the player graphics
playerColorPointer   ds 2           ; pointer to the player colors
duration             ds 1           ; used for success and death sounds
walkingSoundIndex    ds 1           ; index for climber walking sound
startSWCHB           ds 1           ; B&W switch value at game start (reset)
currentSWCHB         ds 1           ; B&W switch value during current frame
lastSWCHB            ds 1           ; B&W switch value last frame
tvType               ds 1           ; holds the NTSC/PAL value (0=NTSC, 1=PAL)
overlay              ds 27          ; overlay place holder (thank you Andrew)

   echo "***",*-$80, "BYTES OF RAM USED", $80 - (*-$80), "BYTES FREE"


   org overlay                      ;[27/27]
;
; the following are needed for the game kernel
;
groupCount           ds 1           ; keeps track of which group of obstacles
                                    ; is being processed during the kernel
playerGraphics       ds PLAYER_HEIGHT; these values are held in RAM for
playerColors         ds PLAYER_HEIGHT; faster processing during the kernel
playerGraphic        ds 1           ; climber graphic for scanline
playerColor          ds 1           ; climber color for scanline
playerBufferedGraph  ds 1           ; buffered climber graphic
playerBufferedColor  ds 1           ; buffered climber color
vecKernel            ds 2           ; indirect jump for the game kernel

   org overlay                      ;[19/27]
;
; the following are needed for the title kernel
;
groupCount           ds 1           ; keeps track of which group to draw
fontHeight           ds 1           ; used for the 6 digit display kernel
graphicGroupPointer  ds 2           ; pointer to the graphic group to draw
graphicsPointers     ds 12          ; loads pointers into RAM
fontColorPointer     ds 2           ; pointer to the graphic color to draw
loopCount            ds 1           ; keeps track of how many times to loop

   org overlay                      ;[5/27]
;
; the following are needed for the game calculations
;
allowedMotion        ds 1           ; allowed movements for the climber
                                    ; basically mimics SWCHA but restricts
                                    ; the climber to walk based on world
                                    ; constraints :)
ladderPointer        ds 4           ; used for ladder calculations


   org overlay                      ;[16/27]
;
; the following are needed for the status kernel
;
livesPointer         ds 4           ; pointer to the lives graphic
levelPointer         ds 12          ; holds the LEVELxx literal for display

   
   SEG code
   org $F000                        ; 4K ROM

Start
;
; Set up everything so the power up state is known.
;
;   sei                             ; No interrupts are used for this game.
                                    ; This is here for clarity purposes and
                                    ; commented out to save 1 byte :)
   cld                              ; clear BCD bit
   ldy INTIM                        ; for random number seed
   
; The next routine comes courtesy of Andrew Davie :) The routine clears all
; variables, TIA registers, and initializes the stack pointer to #$FF. It does
; this in the unusual way of wrapping the stack. Very ingenious!!
   
   ldx #$00
   txs                              ; set stack pointer to #$00
   pha                              ; push accumulator to stack -- stack now
                                    ; points to the beginning (#$FF)
   txa                              ; accumulator now 0
.clear
   pha                              ; clears out all TIA values
   dex                              ; continue until x reaches 0 (255 times)
   bne .clear
   
;----------------------------------------------------------GameInitialization
;
; Initialize the game variables on cart start up. These variables must be
; set for the game to function properly.
;
GameInitialization
   sty random                       ; set the random number seed
   
   lda #%00010001                   ; reflect the playfield and set ball size
   sta CTRLPF                       ; to two clocks
  
   lda #TITLESCREEN                 ; set the present game state to the
   sta gameState                    ; title screen

   asl                              ; a now = 0
MainLoop
;----------------------------------------------------------------VerticalSync
;
; This routine will take care of vertical sync house keeping. Vertical sync
; starts a new television frame. Each frame starts with 3 vertical sync
; lines. These signal to the television to start a new frame.The right
; difficulty switch will determine if the game should use the NTSC or PAL
; color palette. This is checked every new frame.
;
; NOTE:
;  a = 0 when coming into this routine. This saves 2 bytes and 2 cycles during
;  game execution because there's no need to and the SWCHB value to mask all
;  but the right difficulty value
VerticalSync
   asl SWCHB                        ; right difficulty value now in carry bit
   rol                              ; right difficulty value now in D0
   sta tvType                       ; save the value in tvType for later use
   tax                              ; to be used as a look up index
   ldy VerticalBlankTime,x          ; get the appropriate VBLANK time value
   
   lda #$02                         ; D1 = 1
   sta WSYNC                        ; make sure VSYNC happens on a new line
   sta VSYNC                        ; start a new frame
   lsr                              ; accumulator = 0 (Thanks Thomas!)
   sta WSYNC                        ; first line for VSYNC
   sta WSYNC                        ; second line of VSYNC
   sta WSYNC                        ; third and final line of VSYNC
   sta VSYNC                        ; end the VSYNC period
   sty TIM64T                       ; set timer for vertical blank wait
   
;--------------------------------------------------------CheckConsoleSwitches
;
; Check the VCS's console switches. The right difficulty switch will determine
; if the game should use the NTSC or PAL color palette.
;
CheckConsoleSwitches SUBROUTINE
   inc frameCount                   ; frame count is update every frame
   bne .skipAttractMode             ; if it didn't roll over then skip
                                    ; attract mode logic
   lda attractMode                  ; if already in attract mode then skip
   bmi .skipAttractMode             ; incrementing the attract mode counter
   inc attractMode                  ; attract mode updated every 255 frames
                                    ; until it reaches #$80
.skipAttractMode
   lda SWCHB                        ; read the console switch value
   tay                              ; store it in y for later
   and #BW_MASK                     ; mask the B&W switch
   sta currentSWCHB                 ; and store the current state
   
   cmp lastSWCHB                    ; if the B&W value didn't change skip
   beq .checkConsoleSwitches        ; pause check
   
   cmp startSWCHB                   ; again if the B&W value didn't change
   bne .checkConsoleSwitches        ; skip pause check
   
   bit attractMode                  ; if not in attract mode then set the
   bpl .setPausedState              ; pause state of the game
   
   ldy #$00                         ; attract mode was enabled so re-enable
   beq .enableTIA                   ; TIA output
.setPausedState   
   lda gameState                    ; get the current game state
   eor #GAMEPAUSED                  ; EOR will switch the bits to set the
   sta gameState                    ; game mode to pause or running
.checkConsoleSwitches
   tya                              ; lda with SWCHB value
   lsr                              ; move the reset value to the carry bit

   bcs .skipReset                   ; reset button not pressed
   lda #START_NEW_GAME              ; set game state to starting new game
   sta gameState
   ldy #$00                         ; reset attract mode
   beq .enableTIA                   ; same as jmp but saves a byte :)

.skipReset
   tay                              ; move console switch value for later use
   lda frameCount                   ; 
   and #SELECT_DELAY                ; the select switch is checked ~ every 60
   bne .checkSelectSwitch           ; frames or ~ every second
   sta selectDebounce               ;
.checkSelectSwitch
   tya                              ; get the console switch value
   lsr                              ; move the select value to the carry bit
   bcs .continueConsoleSwitchCheck  ; select switch not down so continue
                                    ; vertical sync
.selectSwitchDown
   bit selectDebounce               ; if bit 7 is set high then ignore the
   bmi .continueConsoleSwitchCheck  ; select switch value
.continueSelectSwitchDown
;
; was originally...
;
;   lda #$FF                         ; show that the select switch is held
;   sta selectDebounce               ; for the next frame
;   ldy #$00                         ; to be used to clear the attract timer

;
; suggested by Thomas Jentzsch (saves 1 byte)
;

   ldy #$FF                         ; show that the select switch is held for
   sty selectDebounce               ; the next frame
   iny                              ; y now 0 to clear the attract timer
   lda attractMode                  ; 
   sty attractMode                  ; clear the attract mode timer
   sty frameCount
   bmi .continueConsoleSwitchCheck  ; if currently in attract mode continue
                                    ; with vertical sync
   bit gameState                    ; if currently on the title screen then
   bmi .incrementGameSelection      ; increase the game selection
   
   lda #TITLESCREEN                 ; game select was pressed and recorded so
   sta gameState                    ; put the user back to the title screen
   bmi .continueConsoleSwitchCheck  ; the game selection is only increased
                                    ; while on the title screen so continue
                                    ; checking the console switches
.incrementGameSelection
   lda gameSelection                ; get the current game variation
   eor #TRAINING_MODE               ; EOR will switch the bits to set game
   sta gameSelection                ; selection for training or normal
  
.continueConsoleSwitchCheck
   lda gameState                    ; if the game is over then end the
   cmp #GAME_OVER_STATE             ; console switch check
   beq .endConsoleSwitchCheck
   
   lda frameCount                   ; check the fire button ~every 60 seconds
   and #SELECT_DELAY
   beq .checkJoystick               ; the joystick is checked to determine if
                                    ; the attract mode should be turned off
.checkFireButton
   ldy #$00                         ; used to reset the attract mode timer
   bit INPT4                        ; bit 7 of INPT4 is set high so the fire
   bmi .clearFireButtonDebounce     ; button is not depressed
;
; The fire button was pressed. Check to see if it was held down. If so then
; ignore the button press. If the button press is valid (it wasn't held down
; and enough time has elapsed) then set the game state accordingly.
;
   lda #$FF                         ; check if the fire button is down from
   cmp fireButtonDebounce           ; the previous frame
   beq .checkJoystick               ;
   sta fireButtonDebounce           ; show the fire button was depressed
   ldx gameState                    ; if the game is paused reset attract
   bpl .enableTIA                   ; timer and enable TIA output
   
   lda #START_NEW_GAME              ; the game is on the menu so start a
   sta gameState                    ; new game
   bne .enableTIA
   
.clearFireButtonDebounce
   inc fireButtonDebounce           ; show the fire button was not pressed
   
.checkJoystick
   lda SWCHA                        ; if the joystick was moved turn off
   cmp #$FF                         ; attract mode (enable TV output)
   beq .endConsoleSwitchCheck
   
.enableTIA
   lda levelBCD                     ; if the level is not 0 then we must be
   bne .endConsoleSwitchCheck       ; cycling between title and game over
   sty attractMode                  ; clear the attract mode timer
   sty frameCount                   ; reset the frame count
   sty levelBCD
   
.endConsoleSwitchCheck
   lda currentSWCHB                 ; get the current B&W value
   sta lastSWCHB                    ; and store it in last B&W value for
                                    ; next frame
                                    
   lda gameState                    ; get the current game state
   cmp #GAME_OVER_STATE             ; if the game is over cycle between the
   bne .checkForNewGameState        ; title screen and game over screen
   lda attractMode                  ; once D7 has gone high switch to the
   bpl .doneGameCalculations        ; title screen
   
   jsr SetAttractModeFor30Seconds
   
   lda #TITLESCREEN                 ; set the game state to the title screen
   sta gameState
   bmi TitleScreenCalculations      ; same as jmp but saves a byte :)

.checkForNewGameState
   cmp #START_NEW_GAME              ; if we're not starting a new game then
   bne .continueCheckingGameState   ; continue checking

   jsr StartNewGame                 ; start a new game
.continueCheckingGameState
   bit gameState                    ; bit will set N, V, and Z status
   bpl GameCalculations             ; game is in progress (fall through if not)

;-----------------------------------------------------TitleScreenCalculations
;
; Set up the players for a 6 digit display. They will be centered on the
; screen for the title screen information.
;
TitleScreenCalculations
   ldx tvType                       ; 0 = NTSC, 1 = PAL
   lda TitleScreenBackground,x      ; get the background color for the title
   sta COLUBK
   lda TitleScreenWoodColor,x       ; get the playfield color for the title
   sta COLUPF

;
;   lda #NTSC_SKY_BLUE               ; set the NTSC colors for the title
;   ldx #NTSC_WOOD_BROWN
;   bit SWCHB                        ; check the right difficulty switch
;   bpl .setTitleScreenColors        ; if set to B then use NTSC settings
;   lda #PAL_SKY_BLUE                ; else use PAL settings
;   ldx #PAL_WOOD_BROWN
;.setTitleScreenColors
;   sta COLUBK
;   stx COLUPF
   
   ldx #$06                         ; used to coarse move the players
   jsr RandomByte                   ; called just to re-seed the random number
   sta WSYNC                        ; wait for next scan line
.coarseMovePlayers
   dex
   bpl .coarseMovePlayers
   nop
   sta RESP0                        ; set to cycle 39 or pixel 117 (39 * 3)
   sta RESP1                        ; set to cycle 42 or pixel 126 (42 * 3)
   
; was originally...
;  lda #HMOVE_R3
;  sta HMP0                         ; move right 3 pixels
;  lda #HMOVE_R2
;  sta HMP1                         ; move right 2 pixels

;
; suggested by Thomas Jentzsch (saves 1 byte) BEAUTIFUL!!
;
   lda #HMOVE_R2 | $08              ; = $E8 (%11101000) only top nibbles used
   sta HMP1                         ; move right 2 pixels
   asl                              ; = $D0 (%11010000)
   sta HMP0                         ; move right 3 pixels
   
   sta WSYNC                        ; next scan line
   sta HMOVE                        ; move the players
   
   lda #>TitleDisplayKernel         ; set for an indirect jump in the game
   sta vecKernel+1                  ; kernel
   lda #<TitleDisplayKernel
   sta vecKernel
   
   lda #$03
   sta NUSIZ0                       ; 3 copies of GRP0 close
   sta NUSIZ1                       ; 3 copies of GRP1 close
   
   sta VDELP0                       ; vertical delay for GRP0 (D0=1)
   sta VDELP1                       ; vertical delay for GRP1 (D0=1)
   
   sta REFP0                        ; set the players not to reflect so the
   sta REFP1                        ; title screen displays properly (D3=0)
   
   lda levelBCD                     ; if cycling through game over and title
   beq .resetAttractMode            ; screen level will not be zero
   
   bit attractMode                  ; check attract mode for screen "flip time"
   bpl .gotoDisplayKernel           ; if no attract mode display the kernel
   
   lda #GAME_OVER_STATE             ; set game state to game over so we know to
   sta gameState                    ; "flip" the screen
   jsr SetAttractModeFor30Seconds
   
.resetAttractMode
   sta attractMode                  ; reset the attract mode
.gotoDisplayKernel
   sta HMCLR                        ; clear horizontal movement
   jmp DisplayKernel

;------------------------------------------------------------GameCalculations
;
; The players will be moved in this routine. The sound will be done during
; overscan calculations.
;
GameCalculations
   bvc .decrementGirderSpeed        ; if the game is not paused then move the
                                    ; players (bit gameState from above)
   ldy #$FF                         ; set the allowed motion to no motion
   sty allowedMotion                ; y used so the iny will set it to zero
   jsr TurnOffSounds
.doneGameCalculations
   jmp DoneGameCalculations         ; game paused so nothing to do

.decrementGirderSpeed
   ldy tvType
;   lda SWCHB
;   and #%10000000
;   asl
;   rol
;   tay

   lda levelBCD                     ; get the current level number
   cmp #MAX_LEVEL
   bcc .setGirderSpeed              ; if less than MAX_LEVEL set girder speed
   lda #MAX_LEVEL                   ; keep the girder speed constant
.setGirderSpeed
   asl                              ; multiply by 2 (NTSC/PAL tables merged)
   ora tvType                       ; D0 now holds tvType
   tay                              ; move to y for table index
   
   lda playerMotion                 ; get the girder fractional delay value
   clc
   sbc GameSpeedTable-2,y           ; subtract by 2 because rol multiplied by 2
   sta playerMotion                 ; set the girder fractional delay value
   bcs CheckPlayerState             ; didn't cross 0 -- don't check girder move
;
; Move the obstacles horizontally. This routine is used courtesy of Piero
; Cavina. It is based on his PCMSD demo which eventually became Oyston. His
; original PCMSD source can be found at
; http://www.biglist.com/lists/stella/archives/199704/msg00015.html.
; His game Oystron (great game!) can be purchased at 
; http://www.atariage.com/store
; Visit http://atariage.com/software_page.html?SoftwareLabelID=869 to learn
; more about the game :)
;
MoveGirders
   lda #$00
   sta attractMode                  ; reset the attract mode
   ldx #NUMGRP-1                    ; index for the number of girders
.moveGirdersLoop
   lda startCount,x                 ; if the start count is zero then move
   beq .moveGirder                  ; the corresponding girder
   dec startCount,x                 ; decrement the start count until it
   bpl .checkNextGroup              ; reaches zero
.moveGirder
   lda horPosP1,x                   ; get girder's horizontal position
   cmp #XMAX                        ; if the girder hasn't reached right edge
   bcc .moveGirderRight             ; then continue moving right
   jsr RandomByte                   ; get a random start count for the girder
   and #$0F
  
CalculateFairStartCount
   sta startCount,x
   ldy startCount-1,x               ; check the start count of the adjacent
                                    ; girder
   sty temp                         ; to be used in calculation
   bne .adjustViaStartCount         ; not moving -- compute offset from start
                                    ; count
                                    
   lda horPosP1-1,x                 ; get the adjacent girder's position
   sec
   sbc #XMIN                        ; subtract by XMIN -- this is to be used to
   sta temp                         ; help the girders have a "fair" gap for
                                    ; climber to squeeze through
   lda startCount,x                 ; reset a with startCount
   
.adjustViaStartCount
   sec
   sbc temp                         ; distance between values
   bcs .greaterThan
   eor #$FF                         ; subtraction made a < 0 so make a > 0
   adc #$01                         ; carry was cleared from subtraction
.greaterThan
   cmp #FAIR_PIXEL_DELTA
   bcs .wrapGirderToLeft            ; if distance > fair pixel value continue
   
   lda #FAIR_PIXEL_DELTA
   cmp temp                         ; if the adjacent startCount < fair pixel
   bcs .incrementStartCount         ; delta then increment start count
   eor #$FF                         ; make the number negative so we decrement
   adc #$01                         ; carry was cleared from compare
.incrementStartCount
   clc
   adc temp
   sta startCount,x                 ; make the start count +/- FAIR_PIXEL_DELTA

.wrapGirderToLeft
   lda #XMIN                        ; wrap girder around to the left side of
   sta horPosP1,x                   ; the screen
   bne .checkNextGroup
.moveGirderRight
   inc horPosP1,x                   ; move the girder one pixel to the right
.checkNextGroup
   dex
   bpl .moveGirdersLoop

CheckPlayerState SUBROUTINE
;
; suggested by Andrew Davie (saves 4 bytes!!)
; moving NTSC/PAL selection to RAM saves another byte
;
   lda levelBCD                     ; get the current level number
   cmp #MAX_LEVEL+1
   bcc .setClimberSpeed             ; if less than MAX_LEVEL set climber speed
   lda #MAX_LEVEL-1                 ; decrease climber speed to be slower than
                                    ; the girders
.setClimberSpeed
   asl                              ; multiply by 2 (NTSC/PAL tables merged)
   ora tvType                       ; D0 now holds tvType
   tay                              ; move to y for table index
   
   lda climberMotion                ; get the climber's fractional delay value
   clc
   sbc GameSpeedTable-2,y           ; subtract by 2 because rol multiplied by 2
   sta climberMotion                ; set the climber fraction delay value
   bcs .doneGameCalculations        ; not time to move climber -- do nothing
   
   inc gameClock                    ; gameClock used for player animation
   bit playerState                  ; check the current player state if not
   bpl CheckJoystickValues          ; doing death animation check joysticks
;
; death animation
;
   lda allowedMotion                ; if the climber is not a walkway he is
   and #DOWN_MOTION_MASK            ; descended until he hits the walkway
   bne .doDeathAnimation            ; same as jmp but save 1 byte
   
   inc verPosP0                     ; move the climber down 1 pixel
.doDeathAnimation
   lda gameClock                    ; get the game clock for player animation
;   
; was originally...
;   lsr                              ; updated every other frame
;   and #$01                         ; mask all but D0 alternate between 0 and 1
;

;
; suggested by Andrew Davie (saves 1 byte)
;
   and #$02
   tax                              ; move value to x for table index
   lda DeathAnimationTable,x        ; load the climber death animation
   sta playerPointer                ; into the player pointers
   lda DeathAnimationTable+1,x      ; they will be used to load RAM with the
   sta playerPointer+1              ; death animation
;
; was originally...
;
;   lda tvType                       ; 0 = NTSC, 1 = PAL
;   asl                              ; multiply by 2 for a word read
;   tax
;   ldy DeathAnimationColorTable,x   ; y holds the low byte of the color table
;   lda DeathAnimationColorTable+1,x ; a holds the high byte of the color table

;
; suggested by Thomas Jentzsch (saves 2 bytes)
;
   ldx tvType                       ; 0 = NTSC, 1 = PAL
   ldy DeathAnimationColorTable,x
   lda DeathAnimationColorTable+2,x

   jmp .setVerticalColorPointer     ; set the color pointers
   
CheckJoystickValues
   lda lives                        ; get the number of lives remaining
   bpl .checkAllowedMotion          ; lines >=0 check climber allowed motion
   
;   lda #$FF                         ; the game is over so set the allowed
   sta allowedMotion                ; motion to no motion allowed
.checkAllowedMotion
   lda gameClock                    ; get the game clock for walking sound
   lsr                              ; to be done every other frame

   and #$03                         ; mask all but D0 and D1
   sta walkingSoundIndex
   
   lda SWCHA                        ; get the real joystick value
   ora allowedMotion                ; combine it with allowed motion
;
; was originally...
;   cmp #$FF                         ; if no motion allowed turn off walking
;   bne .checkJoystickMovement       ; sound
;   lda #$00
;   sta walkingSoundIndex

;
; suggested by Thomas Jentzsch (saves 2 bytes)
;
; The eor is great here! The trick here is if eor #$FF on 0 gives #$FF and eor
; on #FF gives 0! What magic :) This save from setting the accumulator to 0 for
; the walkingSoundIndex.
   eor #$FF
   bne .checkJoystickMovement       ; if the player is not pushing the joystick
   sta walkingSoundIndex            ; then turn off the walking sound
   
.checkJoystickMovement

   lda allowedMotion                ; allowedMotion was determined during overscan
   bmi .checkForLeftJoystickMove
   
   lda horPosP0                     ; if the climber has reached his right
   cmp #CLIMBER_X_START             ; most edge then don't move him
   bcs .doneHorizontalCalculations  ; same as jmp but saves a byte
   
.moveClimberRight
   inc horPosP0                     ; move the climber right 1 pixel
   lda #$0F                         ; make the climber face right
   sta playerReflect                ; keep the reflect value of the climber
   bne .doneHorizontalCalculations  ; same as jmp but saves a byte
   
.checkForLeftJoystickMove
   bit allowedMotion                ; bit instead of lda so bvs can be done
   bvs .checkForDownJoystickMove

   lda horPosP0                     ; if the climber has reached his left
   cmp #XMIN+5                      ; most edge then don't move him
   bcs .moveClimberLeft
   
   bcc .doneHorizontalCalculations  ; same as jmp saves a byte
   
.moveClimberLeft
   dec horPosP0                     ; move the climber left 1 pixel
   ldx #$00                         ; make the climber face left
   stx playerReflect                ; and keep the reflect value for later
   beq .doneHorizontalCalculations
   
.checkForDownJoystickMove
   lda allowedMotion
   and #DOWN_MOTION_MASK

   bne .checkForUpJoystickMove      ; player not pushing joystick down

   lda verPosP0                     ; if the climber has reached his lower
   cmp #YMAX                        ; bound then don't move him
.doneGameCalculations
   bcs DoneGameCalculations
   
.moveClimberDown
   inc verPosP0                     ; move the climber down 1 pixel
   bne .doneVerticalCalculations    ; verPosP0 will never be zero (save 1 byte)
   
.checkForUpJoystickMove
   lda allowedMotion
   and #UP_MOTION_MASK
   
   bne DoneGameCalculations         ; player not pushing joystick up
   
   lda verPosP0                     ; if the climber has reached his upper
   cmp #YMIN                        ; bound then don't move him
   beq DoneGameCalculations

.moveClimberUp
   dec verPosP0                     ; move climber up 1 pixel
   
.doneVerticalCalculations  
   ldx #$00                         ; don't reflect the climber when moving
   stx playerReflect                ; vertically
   lda gameClock                    ; get the game clock for player animation
   lsr                              ; updated every other frame
   and #$02
   tax

   lda VerticalMovementTable,x
   sta playerPointer
   lda VerticalMovementTable+1,x
   sta playerPointer+1
   
;   lda LowVerticalMovementTable,x   ; load the pointers to the animation frame
;   sta playerPointer                ; data
;   lda HighVerticalMovementTable,x
;   sta playerPointer+1
   
   ldx tvType                       ; 0 = NTSC, 1 = PAL
   ldy VerticalColorTable,x         ; get the pointers for the vertical color
   lda VerticalColorTable+2,x       ; table
   
.setVerticalColorPointer
   sta playerColorPointer+1
   sty playerColorPointer
   bne DoneGameCalculations
   
.doneHorizontalCalculations
   lda gameClock                    ; get the game clock for player animation
   and #$03                         ; 4 frames of horizontal animation
   tax
   lda LowHorizontalMovementTable,x ; load the pointers to the animation frame
   sta playerPointer                ; data
   lda HighHorizontalMovementTable,x
   sta playerPointer+1
   
   ldx tvType                       ; 0 = NTSC, 1 = PAL
   lda HorizontalColorTable,x
   sta playerColorPointer
   lda HorizontalColorTable+2,x
   sta playerColorPointer+1
   jmp .checkForDownJoystickMove
   
DoneGameCalculations
   lda playerReflect                ; player reflect held in RAM because level
   sta REFP0                        ; data for kernel will change it
   
;
; note -- y is loaded with PLAYER_HEIGHT and the LSB of the graphic and color
; tables are subtracted by one so y will be 0 when the loop is done. This will
; save a ldy #$00 (2 bytes) to clear the player graphics and colors for this
; frame
;
   ldy #PLAYER_HEIGHT
.loadPlayerGraphics                 ; load the player graphics and player
   lda (playerPointer),y            ; colors into RAM used for faster kernel
   sta playerGraphics-1,y           ; processing
   lda (playerColorPointer),y
   sta playerColors-1,y
   dey
   bne .loadPlayerGraphics
   
   bit playerState                  ; if the climber collided with anything
   bmi .positionClimber             ; don't play walking sound
   lda gameState                    ; also if the game is over don't play the
   cmp #GAME_OVER_STATE             ; walking sound
   beq .positionClimber
   asl                              ; shift player state left to check for
   bmi .positionClimber             ; pause state (no walking sound)
   jsr PlayWalkingSound
.positionClimber
   sta WSYNC                        ; make sure this is done on a new scan line
   lda #NUMGRP                      ; subtraction of player horizontal postion
   sta groupCount                   ; must happen on cycle 18 so movement comes
                                    ; out right
   lda #LADDER_COLOR                ; since the game is in progress set the
   sta COLUPF                       ; playfield color to the ladder color (same
                                    ; for NTSC and PAL)
   bit $00                          ; waste 3 cycles
   lda horPosP0
   sec
.coarseMoveClimber
   sbc #$0F
   bcs .coarseMoveClimber
   sta RESP0                        ; set climber's coarse position
   
   sta WSYNC                        ; wait for next scan line
;-------------------------
   sta HMOVE
   tax                              ; a is now used as an offset to load the
   lda HorizontalMoveTable-240,x    ; players fine movement value
;
; note -- y = 0 from .loadPlayerGraphics
;
   sty NUSIZ0                       ; set the size of the players to 1 clock
   sty NUSIZ1
   sty RESBL                        ; set the ball coarse position (pixel 60)
   sty VDELP0                       ; don't vertically delay climber
   sty playerGraphic                ; clear the player graphic and color for
   sty playerColor                  ; this frame
   sta HMP0                         ; set fine movement of climber
   lda #HMOVE_R3
   sta HMBL                         ; set fine movement of ball (pixel 63)

   sta WSYNC                        ; wait for next scan line
;-------------------------
   sta HMOVE

   lda levelBCD
   cmp #MAX_LEVEL
   bcc .setLevelColors
.decrementLevelLoop
; was originally...
;   sec
;   sbc #MAX_LEVEL+1
;   cmp #MAX_LEVEL+1
;   bcs .decrementLevelLoop
;   tax

;
; suggested by Thomas Jentzsch (saves 1 byte) BEAUTIFUL!!
;
   sed                              ; set for decimal mode
   sbc #MAX_LEVEL+1                 ; this routine gets a number between the
   bcs .decrementLevelLoop          ; MAX_LEVEL limit for the color cycling
   adc #MAX_LEVEL+1                 ; add back in because number < 0
   cld                              ; clear decimal mode

.setLevelColors
   tax                              ; move the new value to x for a table index
   ldy GameBackgroundColors-1,x     ; load the background colors
   sty backgroundColor
;
; was originally...
;   bit SWCHB                        ; check for NTSC or PAL palette
;   bmi .setPALLevelColors
;
;   ldy NTSCPlatformColors,x         ; load y sith the platform color
;   ldx #NTSC_GIRDER_COLOR           ; load x with the color of the girders
;   bne .setGameColors               ; implicit jump to save a byte
;.setPALLevelColors
;   ldy PALPlatformColors,x          ; load y with the platform color
;   ldx #PAL_GIRDER_COLOR            ; load x with the color of the girders
;.setGameColors   
;   sta backgroundColor              ; store the background color and
;   sty platformColor                ; platform color in RAM for kernel use
;   stx COLUP1                       ; color the girders

;
; using a suggestion from Andrew Davie about the SetGirderSpeed routine this
; saves 5 bytes (including table data)
;
   
   ldx tvType                       ; 0 = NTSC, 1 = PAL
   ldy GirderColors,x               ; load the girder colors for player GRP1
   sty COLUP1
   
   asl                              ; multiply by 2 (color values merged)
   ora tvType                       ; now D0 = NTSC/PAL
   tax                              ; used for table look up
   lda PlatformColors-2,x           ; load the platform colors
   sta platformColor
   
   lda #$00
;
; This a modified version of Thomas' skipDraw. x is set during VBLANK as an
; offset for the player. During the kernel x will be checked to see if the
; player needs to be drawn. This elimates the use of x in the game kernel
; for anything other than drawing the main player.
;
   sec
   sbc verPosP0                     ; subtract climber's vertical position
   adc #PLAYER_HEIGHT               ; add the climber's height
   tax                              ; x is now an offset to climber's position
   
   lda #>StartGameKernel            ; set the vecKernel to do an indirect jump
   sta vecKernel+1                  ; in the game kernel
   lda #<StartGameKernel
   sta vecKernel
   
   ldy groupCount                   ; get the groupCount
   sty VDELP1                       ; vertically delay the girders
   
   lda backgroundColor              ; get the background color for this frame
   sta COLUBK
   sta HMCLR                        ; clear all horizontal movement
   
;---------------------------------------------------------------DisplayKernel
;
; The screen must be updated each frame on the 2600. Here the program waits
; for vertical blank to end and then jumps to the appropriate display kernel
; (i.e. GameKernel or TitleDisplayKernel). These values were set during the
; vertical blank.
;
DisplayKernel
   ldy INTIM                        ; big timer wait
   bne DisplayKernel
   
   sty WSYNC                        ; end the current scan line
;-------------------------
   sty HMOVE
   
   bit attractMode                  ; check the attract mode if D7 low
   bpl .enableTIA                   ; enable TIA output
   ldy #$02                         ; D7 high so disable TIA output
.enableTIA
   sty VBLANK                       ; set TIA value for output
   jmp (vecKernel)                  ; indirect jump to the appropriate kernel
   
;--------------------------------------------------------------------Overscan
;
Overscan
;
; was originally...
;   lda #NTSC_OVERSCAN_TIME          ; assume NTSC display
;   sta WSYNC                        ; end last scanline
;   sta VBLANK                       ; turn off TIA
;
;   bit SWCHB                        ; check the right difficulty switch
;   bpl .setOverscanTime             ; if set to B then use NTSC settings
;   lda #PAL_OVERSCAN_TIME           ; else use PAL settings
;.setOverscanTime
;   sta TIM64T                       ; set the timer for the "big wait"

;
; using NTSC/PAL setting in RAM shaves off 2 bytes
;
   ldx tvType                       ; 0 = NTSC, 1 = PAL
   lda OverscanWaitTime,x           ; get the value for the overscan wait time
   sta WSYNC                        ; end the last scanline
   sta VBLANK                       ; tuen off TIA (D1=1)
   sta TIM64T                       ; set the timer for the "big wait"

   lda gameState                    ; get the current game state
   bne .overscanWait                ; if the game isn't being played then
                                    ; jump to the big wait -- notice this
                                    ; branches to another bne to save 1 byte
DetermineClimberAllowedMotion
   ldy #NUMGRP-1                    ; cycle through all platforms to see if the
   lda verPosP0                     ; climber is "standing" on a platform
.determineWalkwayLoop
   cmp WalkwayValues,y
   beq .walkwayFound                ; the climber is standing on a platform
   dey
   bpl .determineWalkwayLoop
;
; climber not standing on a platform
;
   lda #%11011111
   bit playerState                  ; if the player was hit and no walkway was
   bmi .setAllowedMotion            ; found make the player fall
   
   lda SWCHA                        ; don't allow horizontal movement but allow
   ora #%11000000                   ; vertical movement (climber is between
   bmi .setAllowedMotion            ; platforms)
.walkwayFound
   lda #%11110000                   ; masking to not allow joystick movement
   bit playerState                  ; D7=1 so don't move climber
   bmi .setAllowedMotion
   and SWCHA                        ; allow horizontal movement
   
   sta allowedMotion

; was originally...   
;   tya                              ; y holds the walkway number
;   and #%00000001                   ; mask all but bit 0
;   asl                              ; multiply the value by 2
;   tay                              ; this will now be used as an index for
;                                    ; odd/even walkways
;   ldx #$00
;.storeValidLadderValues
;   lda LadderValuePointer,y         ; load the ladderPointer values to
;   sta ladderPointer,x              ; determine vertical movement
;   iny
;   inx
;   cpx #$04
;   bne .storeValidLadderValues

;
; suggested by Thomas Jentzsch (saves 7 cycles)
;
   tya                              ; y holds the walkway number
   and #%00000001                   ; mask all but bit 0
   asl                              ; multiply the value by 2
   ora #$04                         ; max of four ladders per walkway
   tay                              ; this will now be used as an index for
                                    ; odd/even walkays
   ldx #$03
.storeValidLadderValues
   dey
   lda LadderValuePointer,y         ; load the ladderPointer values to
   sta ladderPointer,x              ; determine vertical movement
   dex
   bpl .storeValidLadderValues
      
   ldy #$13
   lda horPosP0
.determineLadderNumber
   ldx #%00010000                   ; mask value to not allow upward motion
   cmp (ladderPointer),y            ; check to see if the climber can move up
   beq .setVerticalMotion
   ldx #%00100000                   ; mask value to not allow downward motion
   cmp (ladderPointer+2),y          ; check to see if the climber can move down
   beq .setVerticalMotion
   dey
   bpl .determineLadderNumber       ; check all ladder values
   ldx #%00110000                   ; don't allow vertical motion
.setVerticalMotion
   txa
   ora allowedMotion                ; or with allowed motion to get climber
.setAllowedMotion                   ; motion
   sta allowedMotion                ; store the value in RAM to be used during
                                    ; VBLANK processing
CheckPlayerCollisions
   lda CXP0FB                       ; player0/ball collision
   and #%01000000                   ; mask to get P0/BALL collision value
   bne .playerBallCollision

   ldx CXPPMM                       ; player0/player1 collision
   bpl OverscanWait                 ; no collisions so jump to the big wait
   
.playerGirderCollision
   
   bit playerState                  ; get the current player state
   bmi .playDeathSound              ; if climber previously hit -- play death
                                    ; sound
   stx playerState                  ; store CXPPMM in playerState

   lda #$15
   sta soundIndex                   ; serves an index for sound time
   
   jsr TurnOffSounds                ; turn off any game sounds being played
   
   lda #$01                         ; set duration to 1 so it goes off for
   sta duration                     ; this round
   
.playDeathSound
   dec duration                     ; once duration (hold note) reaches 0 then
                                    ; play the next note
.overscanWait
   bne OverscanWait                 ; hold the previous note -- go to big wait
   
.playNextDeathNote
   lda #$0A                         ; set the hold note duration to 10 frames
   sta duration
   ldx soundCounter                 ; used as the sound frequency
   inc soundCounter
   jsr PlaySound                    ; play the current note
   bne OverscanWait                 ; if the soundIndex !=0 then go to big wait
   lda #$00                         ; show the player has not been hit
   sta playerState
   sta walkingSoundIndex
   dec lives                        ; decrease the number of lives
   bpl .continueGame                ; number of lives is negative -- game over
   
   lda #GAME_OVER_STATE             ; set the game state to show game over
   sta gameState
   
   jsr SetAttractModeFor30Seconds   ; set attract mode to cycle between title
                                    ; and game screen ~ every 30 seconds
   bne .turnOffSound                ; implicit jmp (saves 1 byte)
   
.playerBallCollision
   bit playerState                  ; get the current player state
   bvs .playSuccessSound            ; if player previously retrieved ball then
                                    ; play the success sound
   sta playerState                  ; store player/ball collision for later use
   
   jsr TurnOffSounds                ; turn off any game sounds being played
   
   lda #$01                         ; set duration to 1 so it goes off for
   sta duration                     ; this round
   
   lda #$06                         ; serves an index for success sound
   sta soundIndex                   ; values
   
.playSuccessSound
   ldx #$FF                         ; set the allowed motion to not allow the
   stx allowedMotion                ; climber to move
   
   dec duration                     ; once duration (hold note) reaches 0 then
                                    ; play the next note
   bne OverscanWait                 ; hold the previous note -- go to big wait
   
   lda #$0A                         ; set the hold note duration to 10 frames
   sta duration
   
   ldx soundIndex                   ; get the sound value for the frequency
   
   jsr PlaySound                    ; play the current note
   bne OverscanWait                 ; if the soundIndex !=0 then go to big wait
   
   lda gameSelection                ; if not set for training mode then
   bpl .incrementLevel              ; increment the current level
   
   jsr SetGirderGraphics            ; set the girder graphics based on game
                                    ; selection
   ldx #NUMGRP-1
.checkGirderGraphicsLoop
   lda girderGraphics,x             ; check to see if all groups have a girder
   beq .continueGame                ; if not then restart the current level
   dex
   bpl .checkGirderGraphicsLoop
   
.incrementLevel
   lda levelBCD                     ; get the level number (BCD)
   cmp #LEVEL_MAX_OUT               ; if the player is at highest level then
   beq .continueGame                ; don't increase level number
   
   clc                              ; prepare for addition (clear carry)
   sed                              ; set to decimal mode
   adc #$01                         ; increase the value by 1
   sta levelBCD
   cld                              ; clear decimal mode
   
.continueGame
   jsr RestartLevel                 ; restart with the current level number
   
.turnOffSound
   jsr TurnOffSounds
   sta CXCLR                        ; STROBE--clear all collisions

OverscanWait
   lda INTIM
   bne OverscanWait
   jmp MainLoop
   
;------------------------------------------------------------PlayWalkingSound
;
PlayWalkingSound
   ldx walkingSoundIndex
   lda AudioControl,x
   sta AUDC1
   lda VolumeControl,x
   sta AUDV1
   lda WalkingSound,x
   sta AUDF1
   rts
   
AudioControl
   .byte $00,$05
VolumeControl
   .byte $00,$06
WalkingSound
   .byte $00,$1C
   
;----------------------------------------------------------------StartNewGame
;
StartNewGame
   lda #STARTING_LIVES
   sta lives                        ; set the starting number of lives

   ldx #NUMGRP-1
   lda #GAMERUNNING                 ; show the game is running
   sta gameState
   
   sta selectDebounce               ; show select switch is not down (a = 0)
.resetGirderGraphicsLoop
   sta girderGraphics,x             ; clear the girder graphics
   sta AUDC0,x                      ; turn off the sound
   dex
   bpl .resetGirderGraphicsLoop

   sta backgroundColor              ; set the background to black (a = 0)
   
   lda #STARTING_LEVEL
   sta levelBCD                     ; this one is used to display value
   
   jsr SetGirderGraphics
   ldy tvType                       ; 0 = NTSC, 1 = PAL
   lda PlatformColors,y             ; load the start up platform colors and
   sta platformColor                ; set it
   
   jsr RestartLevel                 ; restart the current level which will
                                    ; be STARTING_LEVEL
   sta CXCLR                        ; clear any previous collisions
   rts
   
;---------------------------------------------------------------TurnOffSounds
TurnOffSounds
   lda #$00
   sta soundCounter
   sta AUDV0
   sta AUDV1
   rts
   
;------------------------------------------------------------------RandomByte
;
; The VCS has no random routine so one must be written by the programmer. This
; routine was used by Manuel Polik in his upcoming Star Fire game. He doesn't
; remember where he originally got this routine :)
;
; Upon return from this function, the accumulator will hold a pseudo random
; number.
;
; Andrew Davie suggested that I eor the random value with a variable that gets
; updated during code execution. frameCount is updated each frame during the
; game. This will help ensure the routine answers a random number.
; Thanks for the tip Andrew!!
RandomByte
   lda random
   asl
   asl
   asl
   eor frameCount                   ; done to help ensure a random number
   asl
   rol random
   lda random
   rts

;--------------------------------------------------SetAttractModeFor30Seconds   
SetAttractModeFor30Seconds
   lda #$1E                         ; to cycle between title screen and
   sta frameCount                   ; game over set the attract mode to fire
                                    ; in approximately 30 seconds (i.e. frame
   lda #$7E                         ; count updated each frame so set frame
   sta attractMode                  ; count to 255 - 30 and attract mode to 
                                    ; 127)
   rts
;----------------------------------------------------------NTSCGameSpeedTable
;NTSCGameSpeedTable
;   .byte NTSC_LEVEL_1_SPEED
;   .byte NTSC_LEVEL_2_SPEED
;   .byte NTSC_LEVEL_3_SPEED
;   .byte NTSC_LEVEL_4_SPEED
;   
;;-----------------------------------------------------------PALGameSpeedTable
;PALGameSpeedTable
;   .byte PAL_LEVEL_1_SPEED
;   .byte PAL_LEVEL_2_SPEED
;   .byte PAL_LEVEL_3_SPEED
;   .byte PAL_LEVEL_4_SPEED

;
; suggested by Andrew Davie
;
GameSpeedTable
   .byte NTSC_LEVEL_1_SPEED, PAL_LEVEL_1_SPEED
   .byte NTSC_LEVEL_2_SPEED, PAL_LEVEL_2_SPEED
   .byte NTSC_LEVEL_3_SPEED, PAL_LEVEL_3_SPEED
   .byte NTSC_LEVEL_4_SPEED, PAL_LEVEL_4_SPEED
KernelHiTbl
   .byte <DrawEvenLadderGroup
   .byte <DrawOddLadderGroup
   .byte <DrawLadderGroup
   
KernelLoTbl
   .byte >DrawEvenLadderGroup
   .byte >DrawOddLadderGroup
   .byte >DrawLadderGroup
   
TitleScreenBackground
   .byte NTSC_SKY_BLUE, PAL_SKY_BLUE
   
TitleScreenWoodColor
   .byte NTSC_WOOD_BROWN, PAL_WOOD_BROWN
   
GirderColors
   .byte NTSC_GIRDER_COLOR, PAL_GIRDER_COLOR
   
HighHorizontalMovementTable
   .byte >HorizontalMoveFrame1
   .byte >HorizontalMoveFrame2
   .byte >HorizontalMoveFrame3
   .byte >HorizontalMoveFrame1
   
LowHorizontalMovementTable
   .byte <HorizontalMoveFrame1-1
   .byte <HorizontalMoveFrame2-1
   .byte <HorizontalMoveFrame3-1
   .byte <HorizontalMoveFrame1-1
   
NTSCHorizontalColors 
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN 
   .byte NTSC_LTBLUE
   
PALHorizontalColors  
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN  
   .byte PAL_LTBLUE

NTSCVerticalColors   
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN   
   .byte NTSC_LTBLUE
   
PALVerticalColors   
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN   
   .byte PAL_LTBLUE
   
GirderStartMaskingValue
   .byte $3F,$2F,$1F,$0F
   
;
; This should fall right on the end of page $F4. The org statement is placed
; here so DASM will report an error if it doesn't
;   
   org $F4F0
   
;---------------------------------------------------------HorizontalMoveTable
;
; This table holds HMPx values to adjust the player for fine horizonatal
; movement. This comes from a combination of Manuel Polik and Eric Ball.
;
; This table should be placed at the end of a page boundary to avoid masking
; the upper nibbles.
;
HorizontalMoveTable
   .byte HMOVE_L7
   .byte HMOVE_L6
   .byte HMOVE_L5
   .byte HMOVE_L4
   .byte HMOVE_L3
   .byte HMOVE_L2
   .byte HMOVE_L1
   .byte HMOVE_0
   .byte HMOVE_R1
   .byte HMOVE_R2
   .byte HMOVE_R3
   .byte HMOVE_R4
   .byte HMOVE_R5
   .byte HMOVE_R6
   .byte HMOVE_R7
   .byte HMOVE_R8
   
;----------------------------------------------------------------------DrawIt
;
; Draw the values of the graphicsPointers. This routine was ripped from
; River Raid. It's a little different from the other 6 digit displays since
; it does a HMOVE on each scan line to blank the HMOVE bars.
;
; This routine MUST NOT cross a page boundary.
;
DrawIt
   ldy fontHeight
.drawGraphicsLoop
   lda (graphicsPointers+8),y       ;5
   tax                              ;2
   lda (graphicsPointers+10),y      ;5
   sta WSYNC
;--------------------------
   sta HMOVE                        ;3
   sty fontHeight                   ;3
   sta temp                         ;3
   lda (graphicsPointers),y         ;5
   sta GRP0                         ;3
   lda (graphicsPointers+2),y       ;5    @ end of HBLANK
   sta GRP1                         ;3
   lda (graphicsPointers+4),y       ;5
   sta GRP0                         ;3
   lda (graphicsPointers+6),y       ;5
   ldy temp                         ;3

   sta GRP1                         ;3
   stx GRP0                         ;3
   sty GRP1                         ;3
   sta GRP0                         ;3
   ldy fontHeight                   ;3
   dey                              ;2
   bpl .drawGraphicsLoop            ;2   @ 60 or 61
   
   iny                              ;62
   sty GRP0
   sty GRP1
   sty GRP0

   sta WSYNC
;--------------------------
   sta HMOVE
   rts
   
SelectedOptionColor
   byte NTSC_RED,PAL_RED

HorizontalColorTable
   .byte <NTSCHorizontalColors-1,<PALHorizontalColors-1
   .byte >NTSCHorizontalColors,>PALHorizontalColors
   
VerticalColorTable
   .byte <NTSCVerticalColors-1,<PALVerticalColors-1
   .byte >NTSCVerticalColors,>PALVerticalColors
   
VerticalBlankTime
   .byte NTSC_VBLANK_TIME, PAL_VBLANK_TIME
   
OverscanWaitTime
   .byte NTSC_OVERSCAN_TIME, PAL_OVERSCAN_TIME
   
;---------------------------------------------------------PlatformLadderValue
;
; This table holds the ladder graphic values for the platform. There is an
; entry for each group.
;
; Do not cross a page boundary. Timing is critical.
;
PF1PlatformLadderValue
   .byte $00
   .byte EVEN_LADDER_PF1 | ODD_LADDER_PF1
   .byte EVEN_LADDER_PF1 | ODD_LADDER_PF1
   .byte EVEN_LADDER_PF1 | ODD_LADDER_PF1
   .byte EVEN_LADDER_PF1 | ODD_LADDER_PF1
   .byte EVEN_LADDER_PF1 | ODD_LADDER_PF1
   .byte EVEN_LADDER_PF1
   .byte $00
   
PF2PlatformLadderValue
   .byte $00
   .byte EVEN_LADDER_PF2 | ODD_LADDER_PF2
   .byte EVEN_LADDER_PF2 | ODD_LADDER_PF2
   .byte EVEN_LADDER_PF2 | ODD_LADDER_PF2
   .byte EVEN_LADDER_PF2 | ODD_LADDER_PF2
   .byte EVEN_LADDER_PF2 | ODD_LADDER_PF2
   .byte EVEN_LADDER_PF2
   .byte $00

;
; suggested by Thomas Jentzsch
;
;----------------------------------------------------------------DivisibleBy4
;
; Look up table to decide if a value is divisible by 4. A zero in a possition
; means the value does not satisfy the condition. The y-register holds the
; value in question. Valid values for y are 20 >= y >= 0.
;
;DivisibleBy4
;   .byte $01,$00,$00,$00,$01
;   .byte $00,$00,$00,$01,$00
;   .byte $00,$00,$01,$00,$00
;   .byte $00,$01,$00,$00,$00,$01
   
;------------------------------------------------------LadderAndWalkwayTables
;
; Tables used for the ladder and walkway computations
;
LadderValuePointer
   .byte <OddLadderValues,>OddLadderValues
   .byte <EvenLadderValues,>EvenLadderValues
   .byte <OddLadderValues,>OddLadderValues
   
EvenLadderValues
   .byte $07,$08,$09,$0A,$0B,$30,$31,$32,$33,$34
   .byte $57,$58,$59,$5A,$5B,$80,$81,$82,$83,$84

OddLadderValues
   .byte $1C,$1D,$1E,$1F,$20,$43,$44,$45,$46,$47
   .byte $6C,$6D,$6E,$6F,$70,$70,$70,$70,$70,$70
   
WalkwayValues
   .byte $18,$30,$48,$60,$78,$90,$A8
   
;-----------------------------------------------------------ClimberSpriteData
;
; The sprite data for the Climber object. Notice that they are not stored in
; the ROM upside down. The data is read in to ZP so there is no need for the
; data to be inverted. If the data was read directly from the table in the
; kernel then it would be better for the data to be inverted to save on cycle
; counts and ROM space.
;
ClimberSpriteData
HorizontalMoveFrame1   
   .byte $1C ;|   XXX  |
   .byte $3E ;|  XXXXX |   
   .byte $FE ;|XXXXXXX |
   .byte $3E ;|  XXXXX |
   .byte $5F ;| X XXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $3E ;|  XXXXX |
   .byte $1C ;|   XXX  |
   .byte $3F ;|  XXXXXX|
   .byte $F9 ;|XXXXX  X|
   .byte $3A ;|  XXX X |
   .byte $32 ;|  XX  X |
   .byte $3E ;|  XXXXX |
   .byte $36 ;|  XX XX |
   .byte $36 ;|  XX XX |
   .byte $12 ;|   X  X |
   .byte $36 ;|  XX XX |

HorizontalMoveFrame2   
   .byte $1C ;|   XXX  |
   .byte $3E ;|  XXXXX |   
   .byte $FE ;|XXXXXXX |
   .byte $3E ;|  XXXXX |
   .byte $5F ;| X XXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $3E ;|  XXXXX |
   .byte $1C ;|   XXX  |
   .byte $3E ;|  XXXXX |
   .byte $2B ;|  X X XX|
   .byte $23 ;|  X   XX|
   .byte $72 ;| XXX  X |
   .byte $7E ;| XXXXXX |
   .byte $6E ;| XX XXX |
   .byte $6F ;| XX XXXX|
   .byte $27 ;|  X  XXX|
   .byte $61 ;| XX    X|

HorizontalMoveFrame3 
   .byte $1C ;|   XXX  |
   .byte $3E ;|  XXXXX |   
   .byte $FE ;|XXXXXXX |
   .byte $3E ;|  XXXXX |
   .byte $5F ;| X XXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $3E ;|  XXXXX |
   .byte $9C ;|X  XXX  |
   .byte $FF ;|XXXXXXXX|
   .byte $FF ;|XXXXXXXX|
   .byte $3A ;|  XXX X |
   .byte $76 ;| XXX XX |
   .byte $7E ;| XXXXXX |
   .byte $2F ;| X XXXX |
   .byte $66 ;| XX  XX |
   .byte $07 ;|     XXX|
   .byte $03 ;|      XX|
   
VerticalMoveFrame1
   .byte $1C ;|   XXX  |
   .byte $3E ;|  XXXXX |
   .byte $3E ;|  XXXXX |
   .byte $3E ;|  XXXXX |
   .byte $BE ;|X XXXXX |
   .byte $DC ;|XX XXX  |
   .byte $FE ;|XXXXXXX |
   .byte $73 ;| XXX  XX|
   .byte $37 ;|  XX XXX|
   .byte $13 ;|   X  XX|
   .byte $7B ;| XXXX XX|
   .byte $F2 ;|XXXX  X |
   .byte $FE ;|XXXXXXX |
   .byte $FE ;|XXXXXXX |
   .byte $FE ;|XXXXXXX |
   .byte $6E ;| XX XXX |
   .byte $0E ;|    XXX |
   .byte $04 ;|     X  |
   
VerticalMoveFrame2
   .byte $38 ;|  XXX   |
   .byte $7C ;| XXXXX  |
   .byte $7C ;| XXXXX  |
   .byte $7C ;| XXXXX  |
   .byte $7D ;| XXXXX X|
   .byte $3B ;|  XXX XX|
   .byte $7F ;| XXXXXXX|
   .byte $CE ;|XX  XXX |
   .byte $DC ;|XX XXX  |
   .byte $C8 ;|XX  X   |
   .byte $EE ;|XXX XXX |
   .byte $4F ;| X  XXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $76 ;| XXX XX |
   .byte $70 ;| XXX    |
   .byte $20 ;|  X     |
   
DeathFrame1
   .byte $28 ;|  X X   |
   .byte $42 ;| X    X |
   .byte $14 ;|   X X  |
   .byte $1C ;|   XXX  |
   .byte $3E ;|  XXXXX |   
   .byte $FE ;|XXXXXXX |
   .byte $3E ;|  XXXXX |
   .byte $5F ;| X XXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $3E ;|  XXXXX |
   .byte $1C ;|   XXX  |
   .byte $3F ;|  XXXXXX|
   .byte $F9 ;|XXXXX  X|
   .byte $3A ;|  XXX X |
   .byte $32 ;|  XX  X |
   .byte $BE ;|X XXXXX |
   .byte $FE ;|XXXXXXX |

DeathFrame2
   .byte $14 ;|   X X  |
   .byte $42 ;| X    X |
   .byte $28 ;|  X X   |
   .byte $1C ;|   XXX  |
   .byte $3E ;|  XXXXX |   
   .byte $FE ;|XXXXXXX |
   .byte $3E ;|  XXXXX |
   .byte $5F ;| X XXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $7F ;| XXXXXXX|
   .byte $3E ;|  XXXXX |
   .byte $1C ;|   XXX  |
   .byte $3F ;|  XXXXXX|
   .byte $F9 ;|XXXXX  X|
   .byte $3A ;|  XXX X |
   .byte $32 ;|  XX  X |
   .byte $BE ;|X XXXXX |
   .byte $FE ;|XXXXXXX |
   
;
; Coming into the GameKernel
;  y = current group count
;  a = current background color
;
; The background color is used for the playfield background as well as
; the platform color.

StartGameKernel
   ldy groupCount                   ; reloaded -- trashed during DisplayKernel
GameKernel
   sta WSYNC                        ; wait for next scan line
;--------------------------
   sta HMOVE                        ; place HMOVE bar on left side of screen
   sta COLUBK                       ; set the background color
   lda playerGraphic                ; load climber graphics and color
   sta GRP0                         ; during HBLANK so climber is drawn before
   lda playerColor                  ; screen output is visible
   sta COLUP0
   
   lda PF1PlatformLadderValue,y     ; get the ladder graphics for this platform
   sta PF1                          ; @25 needs to be before cycle 27 :)
      
   lda PF2PlatformLadderValue,y
   sta PF2                          ; @32 needs to be before cycle 38 :)
   
   lda #$00                         ; clear the girder graphic register
   sta GRP1                         ; GRP1 is vertically delayed
   sta playerGraphic                ; clear the climber graphic data in RAM for
   sta playerBufferedGraph          ; later use
      
   inx                              ; modified skipDraw - if carry bit set then
   cpx #PLAYER_HEIGHT               ; x is out of the range for drawing the
   bcs PlatformScanline2            ; climber

   lda playerGraphics,x             ; load the climber graphics for the next
   sta playerGraphic                ; scan line in RAM
   lda playerColors,x               ; ~ 13 cycles left before next scan line
   
PlatformScanline2
   sta WSYNC                        ; wait for next scan line
;--------------------------
   sta HMOVE                        ; place HMOVE bar on the left side of screen
   sta COLUP0                       ; load climber graphics and color during
   lda playerGraphic                ; HBLANK so the climber is drawn before
   sta GRP0                         ; screen output is visible
   
   lda #$00                         ; clear climber graphic data in RAM for
   sta playerGraphic                ; later use
   
   inx
   cpx #PLAYER_HEIGHT
   bcs .bufferPlayerGraphics

   lda playerGraphics,x
   sta playerGraphic
   lda playerColors,x
   sta playerColor
   
.bufferPlayerGraphics
   inx                              ; load climber graphics for scan line after
   cpx #PLAYER_HEIGHT               ; next. the girders are positioned on the
   bcs .positionGirder              ; next scan line so there's no time to load
                                    ; climber graphics during that scan line
   lda playerGraphics,x             ; load them here to save processing time
   sta playerBufferedGraph
   lda playerColors,x
   sta playerBufferedColor

.positionGirder
   lda playerGraphic

   sta WSYNC
;--------------------------
   sta HMOVE
   sta GRP0                         ; draw player graphic for the scan line
   lda playerColor
   sta COLUP0                       ; draw the player color for the scan line
   lda horPosP1-1,y                 ; get horizontal position of the current
                                    ; girder
   sec
;
; the coarse movement routine MUST hit @ cycle 18 to position the girder
; correctly.
; the max the girder can coarse move is 72 cycles (the above 18 + the max of
; 54)
;
.coarseMoveGirder
   sbc #$0F
   bcs .coarseMoveGirder
   sta RESP1

   sta WSYNC
;--------------------------
   sta HMOVE
   tay                              ; y now hold the offset to read fine motion
   lda HorizontalMoveTable-240,y    ; get fine motion value
   tay                              ; move to y for later (HMPx must not be
                                    ; loaded until 24 cycles AFTER an HMOVE
   lda playerBufferedGraph          ; load player 0 graphics and colors with
   sta GRP0                         ; climber data for this scan line
   lda playerBufferedColor
   sta COLUP0
   
   lda #$00
   sta playerGraphic                ; clear climber graphic data in RAM for
   sta playerBufferedGraph          ; later processing

   sty HMP1                         ; set girder fine movement
   
   inx
   cpx #PLAYER_HEIGHT
   bcs LoadGirderGraphics
   lda playerGraphics,x
   sta playerGraphic
   lda playerColors,x
   sta playerColor
LoadGirderGraphics
   
   ldy groupCount
   lda girderGraphics-1,y           ; set the girder graphics for the next
   sta GRP1                         ; scan line (remember girders VDEL'd)
   
   ldy #GRPHEIGHT                   ; set up to draw the ladders
   lda backgroundColor              ; background color will now reflect the
                                    ; game's background color  
   sta WSYNC
;--------------------------           start of ladder groups
   sta HMOVE
   sta COLUBK
   lda playerGraphic
   sta GRP0
   lda playerColor
   sta COLUP0
   lda #$00

   sta PF1                          ; clear playfield data (no ladders for the
   sta PF2                          ; first 4 scan lines of the ladder group
   
   sta playerGraphic                ; clear climber graphic data in RAM
   sta HMP1                         ; clear girder's horizontal movement
   
   inx
   cpx #PLAYER_HEIGHT
   bcs .decrementY1
   
   lda playerGraphics,x
   sta playerGraphic
   lda playerColors,x
   sta playerColor

.decrementY1   
   dey
   
   sta WSYNC
;--------------------------          second scan line of ladder group
   sta HMOVE
   lda playerGraphic
   sta GRP0
   lda playerColor
   sta COLUP0

   lda #$00
   sta playerGraphic
   inx
   cpx #PLAYER_HEIGHT
   bcs .decrementY2
   
   lda playerGraphics,x
   sta playerGraphic
   lda playerColors,x
   sta playerColor
   
.decrementY2
   dey
   
   sty temp                         ; save y index for later
   
   lda groupCount                   ; check group count for proper jump to
   cmp #NUMGRP                      ; draw the ladders
   bne .setKernelJump
   lda #$02                         ; must be in the first group with no ladder
   bpl .storeIndex
   
.setKernelJump
   and #%00000001                   ; mask all but D0 to get 0 = even group
                                    ; 1 = odd group
.storeIndex
   tay                              ; set value to y for table indexing

   sta WSYNC
;--------------------------          third scan line of ladder group
   sta HMOVE
   lda playerColor
   sta COLUP0
   lda playerGraphic
   sta GRP0

   lda KernelLoTbl,y                ; load vecKernel with the subroutine to
   sta vecKernel+1                  ; indirect jmp to
   lda KernelHiTbl,y
   sta vecKernel
   
   ldy temp                         ; restore y to the group scan line count
   
   lda #$00
   sta playerGraphic
   
   inx
   cpx #PLAYER_HEIGHT
   bcs .decrementY3
   
   lda playerGraphics,x
   sta playerGraphic
   lda playerColors,x
   sta playerColor

.decrementY3
   dey
 
   sta WSYNC
;--------------------------          forth scan line of ladder group
   sta HMOVE
   lda playerColor
   sta COLUP0
   lda playerGraphic
   sta GRP0

   jsr DrawP0ForEmptyLadderGroup
   
   dec groupCount
   dey
   jmp (vecKernel)                  ; jump to appropriate ladder group
   
DrawLadderGroup                     ; first ladder group so no ladders
   sta WSYNC
;--------------------------
   sta HMOVE

   lda playerGraphic
   sta GRP0
   lda playerColor
   sta COLUP0

.setPlayerGraphics
   inx
   cpx #PLAYER_HEIGHT
   bcs .skipGRP0Draw

   lda playerGraphics,x
   sta playerGraphic
   lda playerColors,x
   sta playerColor
   
.nextLadderLine
   dey
;
; was originally...
;   lda DivisibleBy4,y               ; see if we've done 4 scan lines
;   beq DrawLadderGroup              ; keep looping until 4 scan lines drawn
;   bne EmptyLadderKernel            ; 4 scan lines drawn so jmp to empty group

;
; suggested by Thomas Jentzsch (saves 20 bytes...no more table look up!!)
;
   tya                              ; see if y is divisible by 4
   and #$03                         ; mask (D7-D2) tada -- Thanks Thomas :)
   bne DrawLadderGroup              ; keep cycling through the ladder group
   beq EmptyLadderKernel            ; 4 scan lines drawn so jmp to empty group
   
.skipGRP0Draw
   lda #$00
   sta playerGraphic
   jmp .nextLadderLine
   
DrawOddLadderGroup
   sta WSYNC
;--------------------------
   sta HMOVE

   lda playerGraphic
   sta GRP0
   lda playerColor
   sta COLUP0
   
   lda #ODD_LADDER_PF1              ; load ladder values for odd ladder group
   sta PF1                          ; then jmp back to first ladder group to
                                    ; load player graphics. TIA will continue
   lda #ODD_LADDER_PF2              ; draw ladders until the value changes so
   sta PF2                          ; there's no need to keep loading them
   
   jmp .setPlayerGraphics           ; jmp back to first ladder group
   
DrawEvenLadderGroup
   sta WSYNC
;--------------------------
   sta HMOVE
   lda playerGraphic

   sta GRP0
   lda playerColor
   sta COLUP0
   lda #EVEN_LADDER_PF1             ; load ladder values for even ladder group
   sta PF1                          ; then jmp back to fist ladder group to
                                    ; load playe graphics. TIA will continue to
   lda #EVEN_LADDER_PF2             ; draw ladders until the value changes so
   sta PF2                          ; there's no need to keep loading them
   
   jmp .setPlayerGraphics           ; jmp back to first ladder group
   
EmptyLadderKernel
   lda groupCount                   ; get the current group count if we are
   cmp #NUMGRP-1                    ; in the top group get ready to enable the
   bcs .enableBall                  ; ball
   lda #LADDER_COLOR
   bne .drawNextScanline            ; same as jmp but saves a byte
.enableBall
   lda #NTSC_BALL_COLOR
   bit SWCHB                        ; kept this in to avoid trashing registers
   bpl .setBallColor
   lda #PAL_BALL_COLOR
.setBallColor
   cpy #$07                         ; if y is less than 7 then draw the ball
   bcc .drawNextScanline
   lda #LADDER_COLOR                ; notice that the ladder color disables the
                                    ; ball (D1=0)
.drawNextScanline
   sta WSYNC
;--------------------------
   sta HMOVE
   sta ENABL                        ; ball enable and playfield color (ladder
   sta COLUPF                       ; color) share the same bits
   lda playerGraphic
   sta GRP0
   lda playerColor
   sta COLUP0
   lda #$00
   sta PF1                          ; clear the playfield registers
   sta PF2
   sta playerBufferedGraph
   
   jsr DrawP0ForEmptyLadderGroup
   
   dey
   
   sta WSYNC
;--------------------------
   sta HMOVE
   lda playerGraphic
   sta GRP0
   lda playerColor
   sta COLUP0
   
   jsr DrawP0ForEmptyLadderGroup

   dey

   sta WSYNC
;--------------------------
   sta HMOVE
   lda playerGraphic
   sta GRP0
   lda playerColor
   sta COLUP0

   inx
   cpx #PLAYER_HEIGHT
   bcs .setP0Graphics

   lda playerGraphics,x
   sta playerBufferedGraph
   lda playerColors,x
   sta playerBufferedColor
   
.setP0Graphics
   jsr DrawP0ForEmptyLadderGroup
   dey

LastEmptyLadderKernel
   sta WSYNC
;--------------------------
   sta HMOVE
   
   lda playerBufferedGraph
   sta GRP0
   lda playerBufferedColor
   sta COLUP0

   dey
   beq .continueGameKernel
   
   jmp (vecKernel)                  ; indirect jmp back to draw the ladders

.continueGameKernel
   sty GRP1                         ; clear the girder graphics
   lda #LADDER_COLOR
   sta COLUPF
   lda #$00
   sta ENABL
   lda platformColor                ; to be used as background color for the
                                    ; platform
   ldy groupCount
   beq ScoreKernel                  ; branch to do the score kernel
   jmp GameKernel                   ; continue looping


DrawP0ForEmptyLadderGroup
   inx
   cpx #PLAYER_HEIGHT
   bcs .skipP0Draw
   
   lda playerGraphics,x
   sta playerGraphic
   lda playerColors,x
   sta playerColor
   rts
   
.skipP0Draw
   lda #$00
   sta playerGraphic
   rts

;
; The game screen has been drawn. Now the player indicators and level number
; must be drawn.
;
; The game screen takes up 168 scan lines
;     (4 lines for platform + 20 lines for group) * 7 groups)
; The kernel is 200 scan lines (for both PAL and NTSC) so there are 32 scan
; lines left to use
;
ScoreKernel
   lda platformColor
   sta WSYNC
;--------------------------
   sta HMOVE
   sta COLUBK
   sty PF1                          ; clear all graphic registers so they don't
   sty GRP0                         ; show in the score kernel
   sty GRP1
   sty ENABL
   sty PF2
   
   ldx #$03
   stx NUSIZ0                       ; 3 copies of GRP0 close
   stx NUSIZ1                       ; 3 copies of GRP1 close
   stx VDELP0

   lda #>Level0                     ; set up the pointers to display the
   sta levelPointer+1               ; LEVEL xx literal
   
   sta RESP0                        ; coarse position player 0
   sta RESP1                        ; coarse position player 1
   
   sta levelPointer+3               ; level literal resides on the same page
   sta levelPointer+5               ; so set the high byte to the same value
   sta levelPointer+7               ; a loop isn't used so we know when to hit
                                    ; HMOVE to draw the black bar on the left
                                    ; of the screen
   
   lda #HMOVE_L1
   sta HMP0                         ; move player 0 left one pixel

   asl                              ; a = $20 or HMOVE_L2   
   sta HMP1                         ; move player 1 left two pixels

   lda #>Blank                      ; store blanks in the last 4 bytes to mask
   sta levelPointer+9               ; the leading zero of the level number
   sta levelPointer+11
   
   lda #<Blank
   sta.w levelPointer+8             ; waste a cycle to make this hit @ cycle 76
;-------------------------
   sta HMOVE
   lda #<Level0                     ; store the low pointers to the LEVEL
   sta levelPointer                 ; literal
                                    ; no looping done so we know when to hit
   lda #<Level1                     ; HMOVE to draw the black bar on the left
   sta levelPointer+2               ; of the screen
   
   lda #<Level2
   sta levelPointer+4
   
   lda #<Level3
   sta levelPointer+6

   lda levelBCD                     ; get the level number in BCD
   and #$0F                         ; mask the upper nibble
   tay                              ; used as an offset for the number graphic
   
   lda NumberTable,y                ; load the number graphic position
   sta graphicsPointers+10          ; store the pointer to the number
 
   lda #BLACK                       ; set the color of the font to black
   sta COLUP0
   sta COLUP1

   sta REFP0                        ; don't reflect the players
   sta REFP1
   
   lda #GAME_TEXT_HEIGHT-1          ; set fontHeight to be used by DrawIt
   sta fontHeight
   sta HMCLR                        ; clear horizontal movement
   
   lda levelBCD                     ; get the level number in BCD again
   lsr                              ; shift the value down to the lower
   lsr                              ; nibble
   lsr
   lsr
   
   tay                              ; used as an offset for the number graphic
   nop                              ; waste 4 cycles
   nop
;-------------------------
   sta HMOVE
   beq .drawLevelInformation        ; if 0 then we leave the space as a blank
   lda NumberTable,y                ; load the number graphic position
   
   sta levelPointer+8               ; store the pointer to the number
.drawLevelInformation
   jsr DrawIt                       ; draw the LEVEL xx literal
   
   ldx lives                        ; get the number of remaining lives
   bmi .drawGameOver                ; if it's rolled over the game is over
   dex                              ; used to show number of remaining lives

   bpl .displayReservedLives        ; always positive
   ldx #$16                         ; set up for the end of the kernel
   bne .loop

.drawGameOver
   ldx #$0B                         ; used as an index to load the graphic
                                    ; pointers
.storeGameOverLoop
   ldy #$03                         ; used to keep track of how long we've
                                    ; been in the loop
;                                    
; the loop to load the graphics would take 169 cycles (2 scan lines) we need
; control of this so we know when to hit the HMOVE to black out the left side
; of the screen. The following loop takes the graphic load and splits it into
; 4 iterations so we know when to hit HMOVE. At most the loop will take 15
; cycles so we can loop for 4 times before we run out of time for the scan.
; (3 * 15) + 14 = 59 cycles + 10 = 69

   sta WSYNC
;-------------------------
   sta HMOVE                  ;3
.gameOverGraphicLoop
   lda GameOverLiteral,x      ;4
   sta levelPointer,x         ;4
   dex                        ;2
   dey                        ;2
   bpl .gameOverGraphicLoop   ;2
   txa                        ;2     used to test x for negative
   bpl .storeGameOverLoop     ;2
   
   sta WSYNC
;-------------------------
   sta HMOVE
   lda #GAME_TEXT_HEIGHT-1
   sta fontHeight
   
   jsr DrawIt                       ; draw the "GAME OVER" literal
   
   ldx #$0C                         ; set up for end of kernel loop
   bne .loop

.displayReservedLives
   sta RESP0                        ; player now at pixel 66
   sty VDELP0                       ; turn off vertical delay
   stx NUSIZ0                       ; x is remaining lives - 1
   lda #%00001000
   sta REFP0                        ; reflect player 0
   
   ldx #RESERVE_LIVES_HEIGHT+1
;
; note -- y = 0 from return of DrawIt routine
;
.drawReservedLives
   sta WSYNC
;-------------------------
   sta HMOVE
   lda HorizontalMoveFrame1,y       ; use the horizontal frame for lives
   sta GRP0                         ; indicator
   iny                              ; graphic is stored in ROM right side up
   dex
   bne .drawReservedLives
   stx GRP0                         ; clear player graphic register
   
   ldx #$0D
.loop                               ; remaining scan lines for the kernel
   sta WSYNC
   sta HMOVE
   dex
   bpl .loop
   jmp Overscan
   
;----------------------------------------------------------------RestartLevel
;
RestartLevel
   ldx #NUMGRP
   ldy levelBCD                     ; level number used to read masking value
   cpy #MAX_LEVEL                   ; table -- this will manipulate the girder
   bcc .readMaskingValue            ; start frequency for a new level
   ldy #MAX_LEVEL
.readMaskingValue
   lda GirderStartMaskingValue-1,y
   sta temp
   ldy #$01                         ; every other start count will be 1   
   
.resetStartCountLoop
   jsr RandomByte                   ; get a random number
   and temp                         ; mask the random number based on level
   cmp #FAIR_PIXEL_DELTA            ; make sure it's greater than fair distance
   bcs .setGroupStartCount
   lda #FAIR_PIXEL_DELTA-1          ; set the start count from fair distance
.setGroupStartCount
   sta startCount-1,x
   lda #XMIN
   sta horPosP1-1,x                   ; position girders on the left
   dex
   beq .setClimberStartPosition     ; x wrapped around then leave routine
   sty startCount-1,x                 ; makes the girder start immediately
   sta horPosP1-1,x
   dex
   bpl .resetStartCountLoop
   
.setClimberStartPosition
   lda #CLIMBER_X_START             ; set the climber on the right
   sta horPosP0
   lda #CLIMBER_Y_START             ; set the climber to the bottom
   sta verPosP0
   
   lda #>HorizontalMoveFrame1       ; set animation frame so the climber is
   sta playerPointer+1              ; standing pointing left ready to go
   
   lda #<HorizontalMoveFrame1-1
   sta playerPointer
   
   stx gameClock                    ; clear the game clock
   stx playerState                  ; clear the climber current state
   stx frameCount                   ; reset the frame count
   stx REFP0                        ; don't refect the climber
   stx playerMotion
   stx climberMotion
   
   ldy tvType
;   bit SWCHB                        ; check for PAL or NTSC setting
;   bpl .setPlayerColors
;   ldx #$02                         ; index for PAL colors
;.setPlayerColors
   lda HorizontalColorTable,y
   sta playerColorPointer
   lda HorizontalColorTable+2,y
   sta playerColorPointer+1
   rts
   
NTSCFontColorTable
   .byte NTSC_WHITE
   .byte NTSC_BROWN
   .byte NTSC_BROWN
   .byte NTSC_WHITE
   .byte NTSC_ATARI_AGE_BLUE
   
PALFontColorTable
   .byte PAL_WHITE
   .byte PAL_BROWN
   .byte PAL_BROWN
   .byte PAL_WHITE
   .byte PAL_ATARI_AGE_BLUE
   
GameBackgroundColors
   .byte BLACK,BLACK-78,BLACK-156,BLACK-234,BLACK-312
PlatformColors
   .byte NTSC_PINK, PAL_PINK
   .byte NTSC_PINK+62, PAL_PINK+62
   .byte NTSC_PINK+124, PAL_PINK+124
   .byte NTSC_PINK+186, PAL_PINK+186
   .byte NTSC_PINK+248, PAL_PINK+248
   
;-----------------------------------------------------------SetGirderGraphics
;
SetGirderGraphics
   ldy #GIRDER_GRAPHIC              ; load with the girder graphic pattern
   bit gameSelection                ; check the game selection for training
   bpl .normalGameSetGirderGraphics ; D7 = 0 so this is a normal game
   
   lda levelBCD                     ; check to see if we're player level 1
   cmp #STARTING_LEVEL
   bne .doneGirderGraphics          ; if not then advance game as normal
   
   jsr RandomByte                   ; get a random number
.setRandomGirderGraphicLoop
   cmp #NUMGRP                      ; if it's less than NUMGRP we're okay
   bcc .determineRandomGraphic
   lda #NUMGRP-1                    ; set to NUMGRP-1 (was greater than NUMGRP)
.determineRandomGraphic
   tax                              ; used an index for girder graphics to show
   lda girderGraphics,x             ; if the girder graphic is empty load it
   beq .storeGirderGraphics
   dex                              ; check next position
   txa
   jmp .setRandomGirderGraphicLoop
   
.storeGirderGraphics 
   sty girderGraphics,x
   rts
   
.normalGameSetGirderGraphics
   ldx #NUMGRP-1
.setGirderGraphicsLoop
   sty girderGraphics,x             ; load with girder graphic pattern
   dex
   bpl .setGirderGraphicsLoop
.doneGirderGraphics
   rts
   
;-------------------------------------------------------------------PlaySound
;
; The volume and channel will be set this routine. The x register must hold
; the frequency. This sound routine only manipulates sound channel 0
;
PlaySound
   lda #GAME_SOUND_VOLUME
   sta AUDV0
   lda #$0C
   sta AUDC0
   stx AUDF0
   dec soundIndex
   rts
   
;----------------------------------------------------------TitleDisplayKernel
;
; The title screen display is drawn here.
;
TitleDisplayKernel SUBROUTINE
   lda #$04
   sta groupCount                   ; groupCount used for table lookup
   sta WSYNC
;--------------------------
   sta HMOVE

.atariAgePresentsKernel
   lsr
   sta loopCount                    ; set loopCount to 2 (cycles to draw 2
                                    ; groups of literals)
   jsr DisplayGraphicsLiteral       ; draw the literals
   
   ldx #$03                         ; 4 blank lines before drawing Climber 5
.climber5Kernel
   sta WSYNC
;--------------------------
   sta HMOVE
   dex
   bpl .climber5Kernel
   
   ldx #$02                         ; set loop count to loop twice
   stx loopCount
   
   
   lda #%11111110                   ; set the playfield data for the "wood"
   sta PF2                          ; background for Climber 5 literal
                                    ; (playfield set to reflect)
.emptyLineLoop1                     ; 3 blank lines with just playfield
   sta WSYNC
;--------------------------
   sta HMOVE
   dex
   bne .emptyLineLoop1
   
   jsr DisplayGraphicsLiteral       ; draw Climber 5 literal

   sta WSYNC
;--------------------------
   sta HMOVE
   
   sta WSYNC
;--------------------------
   sta HMOVE
   
   lda #$00                         ; stop drawing "wood" background for
   sta PF2                          ; Climber 5 literal
   
   ldx #$29                         ; draw 41 blank lines
.skipLinesLoop
   sta WSYNC
;-------------------------
   sta HMOVE
   dex
   bne .skipLinesLoop
   
   inc loopCount                    ; only draw 1 literal (was 0 before here)

   lda #<TrainingLiteral            ; set the graphic pointers to draw the
   sta graphicGroupPointer          ; training literal
   
   lda #>TrainingLiteral
   sta graphicGroupPointer+1
   
   lda #TEXT_HEIGHT-1
   sta fontHeight
   
   lda #NTSC_WHITE                  ; default to a white color (same as PAL)
   sta COLUP0
   sta COLUP1
   
   lda gameSelection                ; get the current game selection
   bpl .drawTrainingSelectionOptions; if not training then draw options
   
   ldx tvType                       ; 0 = NTSC, 1 = PAL
   lda SelectedOptionColor,x        ; load the appropriate red color
   
   sta COLUP0                       ; color the selected game selection
   sta COLUP1
.drawTrainingSelectionOptions
   jsr BeginningGraphicLoop         ; draw Training literal
   
   sta WSYNC
;-------------------------
   sta HMOVE
   
   lda #<NormalLiteral              ; set the graphic pointers to draw the
   sta graphicGroupPointer          ; normal literal
   
   lda #>NormalLiteral
   sta graphicGroupPointer+1
   
   lda #TEXT_HEIGHT-1
   sta fontHeight
   
   lda #NTSC_WHITE                  ; default to a white color
   sta COLUP0
   sta COLUP1
   
   lda gameSelection                ; get the current game selection
   bmi .normalModeFontColorSet      ; if not normal then draw the literal
   
   ldx tvType                       ; 0 = NTSC, 1 = PAL
   lda SelectedOptionColor,x        ; load the appropriate red color
   
   sta COLUP0                       ; color the selected game selection
   sta COLUP1
.normalModeFontColorSet
   inc loopCount                    ; only draw 1 literal (was 0 before this)
   jsr BeginningGraphicLoop         ; draw Normal literal
   
   ldx #$17                         ; skip 23 blank lines
.loop
   sta WSYNC
;-------------------------
   sta HMOVE
   dex
   bne .loop
   
   stx groupCount
   inc loopCount                    ; only draw 1 literal (was 0 before this)
   jsr DisplayGraphicsLiteral       ; draw copyright literal
   
   jmp Overscan                     ; go to overscan for the big wait
      
   
DisplayGraphicsLiteral
   sta WSYNC
;-------------------------
   sta HMOVE                     ;3
   
   ldy groupCount                ;2
   lda GraphicsGroupLow,y        ;4
   sta graphicGroupPointer       ;3
   lda GraphicsGroupHigh,y       ;4
   sta graphicGroupPointer+1     ;3
   lda FontHeightTable,y         ;3
   sta fontHeight                ;3
   
   ldx tvType
   lda FontColorPointerLo,x      ;4
   sta fontColorPointer          ;3
   
   lda FontColorPointerHi,x      ;4
   sta fontColorPointer+1        ;3
   
   lda (fontColorPointer),y      ;5
   
.setFontColors
   sta COLUP0                    ;3
   sta COLUP1                    ;3
   
BeginningGraphicLoop
   ldy #$0B
   ldx #$02

.storeGraphicLoop 
   sta WSYNC
;-------------------------
   sta HMOVE                     ;3
.graphicPointerLoop
   lda (graphicGroupPointer),y   ;5
   sta graphicsPointers,y        ;5
   dey                           ;2
   dex                           ;2
   bpl .graphicPointerLoop       ;2

   ldx #$02
   tya
   bpl .storeGraphicLoop
   
   sta WSYNC
;-------------------------
   sta HMOVE
   
   jsr DrawIt
   dec groupCount
   dec loopCount
   bne DisplayGraphicsLiteral
   rts
   
FontColorPointerHi
   .byte >NTSCFontColorTable
   .byte >PALFontColorTable

FontColorPointerLo
   .byte <NTSCFontColorTable
   .byte <PALFontColorTable
   
GraphicsGroupLow
   .byte <CopyrightLiteral
   .byte <Climber5Literal
   .byte <ClimberLiteral
   .byte <PresentsLiteral
   .byte <AtariAgeLiteral
   
GraphicsGroupHigh
   .byte >CopyrightLiteral
   .byte >Climber5Literal
   .byte >ClimberLiteral
   .byte >PresentsLiteral
   .byte >AtariAgeLiteral
   
AtariAgeLiteral
   .word AtariAgeSymbol0
   .word AtariAgeSymbol1
   .word AtariAge0
   .word AtariAge1
   .word AtariAge2
   .word AtariAge3
   
ClimberLiteral
   .word Climber0
   .word Climber1
   .word Climber2
   .word Climber3
   .word Climber4
   .word Climber5 
   
CopyrightLiteral
   .word Copyright0
   .word Copyright1
   .word Copyright2
   .word Copyright3
   .word Copyright4
   .word Copyright5
   
GameOverLiteral
   .word GameOver0
   .word GameOver1
   .word GameOver2
   .word GameOver3
   .word GameOver4
   .word GameOver5
   
PresentsLiteral
   .word Blank
   .word Presents0
   .word Presents1
   .word Presents2
   .word Presents3
   
Climber5Literal
   .word Blank
   .word Climber50
   .word Climber51
   .word Climber52
   .word Climber53
;
; last word shared with next table so don't cross page boundaries
;   
NormalLiteral
   .word Blank
   .word Normal0
   .word Normal1
   .word Normal2
   .word Normal3
;
; last word shared with next table so don't cross page boundaries
;   
   
TrainingLiteral
   .word Blank
   .word Training0
   .word Training1
   .word Training2
   .word Training3
   .word Blank
   
FontHeightTable
   .byte COPYRIGHT_HEIGHT-1
   .byte NUMBER_5_HEIGHT-1
   .byte CLIMBER_FONT_HEIGHT-1
   .byte PRESENTS_FONT_HEIGHT-1
   .byte AA_FONT_HEIGHT-1
   
NumberTable
   .byte <zero,<one,<two,<three,<four
   .byte <five,<six,<seven,<eight,<nine
   
   echo "***",*-$F000, "BYTES OF ROM USED"
   
   align 256

Climber0
   .byte %00111000;|  XXX   |
   .byte %01111100;| XXXXX  |
   .byte %01111100;| XXXXX  |
   .byte %11101110;|XXX XXX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000000;|XX      |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11101110;|XXX XXX |
   .byte %01111100;| XXXXX  |
   .byte %01111100;| XXXXX  |
   .byte %00111000;|  XXX   |
Climber1
   .byte %11110110;|XXXX XX |
   .byte %11110110;|XXXX XX |
   .byte %11110110;|XXXX XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
   .byte %11000110;|XX   XX |
Climber2
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11001100;|XX  XX  |
   .byte %11111111;|XXXXXXXX|
   .byte %11111111;|XXXXXXXX|
   .byte %01111111;| XXXXXXX|
   .byte %01111111;| XXXXXXX|
   .byte %00110011;|  XX  XX|
Climber3
   .byte %11001110;|XX  XXX |
   .byte %11011111;|XX XXXXX|
   .byte %11011111;|XX XXXXX|
   .byte %11011011;|XX XX XX|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011011;|XX XX XX|
   .byte %11011111;|XX XXXXX|
   .byte %11011110;|XX XXXX |
   .byte %11011111;|XX XXXXX|
   .byte %11011011;|XX XX XX|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011001;|XX XX  X|
   .byte %11011011;|XX XX XX|
   .byte %10011111;|X  XXXXX|
   .byte %10011111;|X  XXXXX|
   .byte %00001110;|    XXX |
Climber4
   .byte %00001110;|    XXX |
   .byte %00011111;|   XXXXX|
   .byte %00011111;|   XXXXX|
   .byte %10011000;|X  XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %10011000;|X  XX   |
   .byte %00011111;|   XXXXX|
   .byte %00011111;|   XXXXX|
   .byte %00011111;|   XXXXX|
   .byte %10011000;|X  XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %11011000;|XX XX   |
   .byte %10011000;|X  XX   |
   .byte %00011111;|   XXXXX|
   .byte %00011111;|   XXXXX|
   .byte %00001110;|    XXX |
Climber5
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01101110;| XX XXX |
   .byte %01111100;| XXXXX  |
   .byte %01111000;| XXXX   |
   .byte %01101100;| XX XX  |
   .byte %01101110;| XX XXX |
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01100111;| XX  XXX|
   .byte %01101110;| XX XXX |
   .byte %01101100;| XX XX  |
   .byte %01111100;| XXXXX  |
   .byte %00111000;|  XXX   |
   
AtariAgeSymbol0
   .byte %00000111;|     XXX|
   .byte %00001000;|    X   |
   .byte %00010111;|   X XXX|
   .byte %00101111;|  X XXXX|
   .byte %01011111;| X XXXXX|
   .byte %01010111;| X X XXX|
   .byte %01011011;| X XX XX|
   .byte %01011101;| X XXX X|
   .byte %01010110;| X X XX |
   .byte %01011011;| X XX XX|
   .byte %01011101;| X XXX X|
   .byte %00101110;|  X XXX |
   .byte %00010111;|   X XXX|
   .byte %00001000;|    X   |
   .byte %00000111;|     XXX|
   
AtariAgeSymbol1
   .byte %11000000;|XX      |
   .byte %00100000;|  X     |
   .byte %11010000;|XX X    |
   .byte %11101000;|XXX X   |
   .byte %11110100;|XXXX X  |
   .byte %11010100;|XX X X  |
   .byte %10110100;|X XX X  |
   .byte %01110100;| XXX X  |
   .byte %11010100;|XX X X  |
   .byte %10110100;|X XX X  |
   .byte %01110100;| XXX X  |
   .byte %11101000;|XXX X   |
   .byte %11010000;|XX X    |
   .byte %00100000;|  X     |
   .byte %11000000;|XX      |
   
Presents0
   .byte %10000000;|X       |
   .byte %10000000;|X       |
   .byte %11101000;|XXX X   |
   .byte %10101000;|X X X   |
   .byte %10101000;|X X X   |
   .byte %10101000;|X X X   |
   .byte %11101110;|XXX XXX |
;
; last empty byte shared with next table so don't cross page boundaries
;   
Presents1
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %11101110;|XXX XXX |
   .byte %10000010;|X     X |
   .byte %11101110;|XXX XXX |
   .byte %10101000;|X X X   |
   .byte %11101110;|XXX XXX |
;
; last empty byte shared with next table so don't cross page boundaries
;   
Presents2
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %11101010;|XXX X X |
   .byte %10001010;|X   X X |
   .byte %11101010;|XXX X X |
   .byte %10101010;|X X X X |
   .byte %11101110;|XXX XXX |
;
; last empty byte shared with next table so don't cross page boundaries
;   
Presents3
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %01101110;| XX XXX |
   .byte %01000010;| X    X |
   .byte %01001110;| X  XXX |
   .byte %01001000;| X  X   |
   .byte %11101110;|XXX XXX |
   .byte %01000000;| X      |

VerticalMovementTable
   .word VerticalMoveFrame1-1
   .word VerticalMoveFrame2-1
   
;HighVerticalMovementTable
;   .byte >VerticalMoveFrame1
;   .byte >VerticalMoveFrame2
;   
;LowVerticalMovementTable
;   .byte <VerticalMoveFrame1
;   .byte <VerticalMoveFrame2

;
; was originally...
;
;HighDeathAnimationTable
;   .byte #>DeathFrame1
;   .byte #>DeathFrame2
;   
;LowDeathAnimationTable
;   .byte #<DeathFrame1
;   .byte #<DeathFrame2

;
; suggested by Andrew Davie
;
DeathAnimationTable
   .word DeathFrame1-1, DeathFrame2-1

AtariAge0
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %10100010;|X X   X |
   .byte %10100100;|X X  X  |
   .byte %10100100;|X X  X  |
   .byte %11100100;|XXX  X  |
   .byte %10101110;|X X XXX |
   .byte %10100100;|X X  X  |
   .byte %01000000;| X      |
;
; last 3 empty bytes shared with next table so don't cross page boundaries
;

AtariAge1
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %01110100;| XXX X  |
   .byte %10010100;|X  X X  |
   .byte %01110100;| XXX X  |
   .byte %00010110;|   X XX |
   .byte %01100101;| XX  X X|
;
; last 5 empty bytes shared with next table so don't cross page boundaries
;
AtariAge2
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00101010;|  X X X |
   .byte %00101010;|  X X X |
   .byte %00101010;|  X X X |
   .byte %10101110;|X X XXX |
   .byte %00101010;|  X X X |
   .byte %00001010;|    X X |
   .byte %00100100;|  X  X  |
;
; last 3 empty bytes shared with next table so don't cross page boundaries
;   
AtariAge3
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %01100000;| XX     |
   .byte %10010000;|X  X    |
   .byte %00010011;|   X  XX|
   .byte %01110100;| XXX X  |
   .byte %10010111;|X  X XXX|
   .byte %01110101;| XXX X X|
   .byte %00000010;|      X |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %00000000;|        |

Climber50
   .byte %00000000;|        |
   .byte %00000001;|       X|
   .byte %00000000;|        |
   .byte %00000001;|       X|
   .byte %00000001;|       X|
   .byte %00000001;|       X|
   
Climber51
   .byte %00111111;|  XXXXXX|
   .byte %11111000;|XXXXX   |
   .byte %00000000;|        |
   .byte %11111111;|XXXXXXXX|
   .byte %11111000;|XXXXX   |
   .byte %11111111;|XXXXXXXX|
   
Climber52
   .byte %11111100;|XXXXXX  |
   .byte %00011111;|   XXXXX|
   .byte %00011111;|   XXXXX|
   .byte %11111100;|XXXXXX  |
   .byte %00000000;|        |
   .byte %11111111;|XXXXXXXX|

Climber53
   .byte %00000000;|        |
   .byte %10000000;|X       |
   .byte %10000000;|X       |
   .byte %00000000;|        |
   .byte %00000000;|        |
   .byte %10000000;|X       |
   
Normal0
   .byte %01001001;| X  X  X|
   .byte %01001010;| X  X X |
   .byte %01011010;| X XX X |
   .byte %01101010;| XX X X |
   .byte %01001001;| X  X  X|
   
Normal1
   .byte %10010010;|X  X  X |
   .byte %01010010;| X X  X |
   .byte %01011100;| X XXX  |
   .byte %01010010;| X X  X |
   .byte %10011100;|X  XXX  |
   
Normal2
   .byte %10001010;|X   X X |
   .byte %10101010;|X X X X |
   .byte %10101011;|X X X XX|
   .byte %11011010;|XX XX X |
   .byte %10001001;|X   X  X|
   
Normal3
   .byte %01011110;| X XXXX |
   .byte %01010000;| X X    |
   .byte %11010000;|XX X    |
   .byte %01010000;| X X    |
   .byte %10010000;|X  X    |
   
   
Copyright0  
   .byte %00111000;|  XXX   |
   .byte %01000100;| X   X  |
   .byte %10111010;|X XXX X |
   .byte %10100010;|X X   X |
   .byte %10111010;|X XXX X |
   .byte %01000100;| X   X  |
   .byte %00111000;|  XXX   |
;
; last empty byte shared with next table so don't cross page boundaries
;
Copyright1
   .byte %00000000;|        |
   .byte %11101110;|XXX XXX |
   .byte %10001010;|X   X X |
   .byte %11101010;|XXX X X |
   .byte %00101010;|  X X X |
   .byte %11101110;|XXX XXX |
   .byte %00000000;|        |
;
; last empty byte shared with next table so don't cross page boundaries
;
Copyright2
   .byte %00000000;|        |
   .byte %11101110;|XXX XXX |
   .byte %10100010;|X X   X |
   .byte %10101110;|X X XXX |
   .byte %10100010;|X X   X |
   .byte %11101110;|XXX XXX |
   .byte %00000000;|        |
;
; last empty byte shared with next table so don't cross page boundaries
;
Copyright3
   .byte %00000000;|        |
   .byte %11010110;|XX X XX |
   .byte %10100101;|X X  X X|
   .byte %10100101;|X X  X X|
   .byte %10100101;|X X  X X|
   .byte %10100101;|X X  X X|
   .byte %10100101;|X X  X X|
   .byte %11000110;|XX   XX |

Copyright4
   .byte %00000000;|        |
   .byte %01110111;| XXX XXX|
   .byte %01000101;| X   X X|
   .byte %01110101;| XXX X X|
   .byte %01010101;| X X X X|
   .byte %01110111;| XXX XXX|
   .byte %00000100;|     X  |
   .byte %00000100;|     X  |

Copyright5
   .byte %00000000;|        |
   .byte %01000111;| X   XXX|
   .byte %01000101;| X   X X|
   .byte %01000101;| X   X X|
   .byte %01000101;| X   X X|
   .byte %01110111;| XXX XXX|
   .byte %00000000;|        |
   .byte %00000000;|        |

Training0
   .byte %01001001;| X  X  X|
   .byte %01001001;| X  X  X|
   .byte %01001110;| X  XXX |
   .byte %01001001;| X  X  X|
   .byte %11101110;|XXX XXX |
   
Training1
   .byte %01001010;| X  X X |
   .byte %01001010;| X  X X |
   .byte %01111010;| XXXX X |
   .byte %01001010;| X  X X |
   .byte %00110010;|  XX  X |
   
Training2
   .byte %10010101;|X  X X X|
   .byte %10010101;|X  X X X|
   .byte %10110101;|X XX X X|
   .byte %11010101;|XX X X X|
   .byte %10010101;|X  X X X|
   
Training3
   .byte %00100111;|  X  XXX|
   .byte %00101001;|  X X  X|
   .byte %01101011;| XX X XX|
   .byte %10101000;|X X X   |
   .byte %00100111;|  X  XXX|

NTSCDeathAnimationColors
   .byte NTSC_WHITE
   .byte NTSC_WHITE
   .byte NTSC_WHITE
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_PEACH
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   .byte NTSC_GREEN
   
PALDeathAnimationColors
   .byte PAL_WHITE
   .byte PAL_WHITE
   .byte PAL_WHITE
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_PEACH
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
   .byte PAL_GREEN
      
DeathAnimationColorTable
   .byte <NTSCDeathAnimationColors-1,<PALDeathAnimationColors-1
   .byte >NTSCDeathAnimationColors,>PALDeathAnimationColors
   
GameOver0
   .byte %00111010;|  XXX X |
   .byte %01001010;| X  X X |
   .byte %01011011;| X XX XX|
   .byte %01000010;| X    X |
   .byte %00111001;|  XXX  X|
   
GameOver1
   .byte %01010001;| X X   X|
   .byte %01010001;| X X   X|
   .byte %11010101;|XX X X X|
   .byte %01011011;| X XX XX|
   .byte %10010001;|X  X   X|
   
GameOver2
   .byte %01111000;| XXXX   |
   .byte %01000000;| X      |
   .byte %01110000;| XXX    |
   .byte %01000000;| X      |
   .byte %01111000;| XXXX   |
   
GameOver3
   .byte %00001100;|    XX  |
   .byte %00010010;|   X  X |
   .byte %00010010;|   X  X |
   .byte %00010010;|   X  X |
   .byte %00001100;|    XX  |
   
GameOver4
   .byte %00100011;|  X   XX|
   .byte %01010010;| X X  X |
   .byte %10001011;|X   X XX|
   .byte %10001010;|X   X X |
   .byte %10001011;|X   X XX|
   
GameOver5
   .byte %11010010;|XX X  X |
   .byte %00010010;|   X  X |
   .byte %10011100;|X  XXX  |
   .byte %00010010;|   X  X |
   .byte %11011100;|XX XXX  |


Level0
   .byte %00011111;|   XXXXX|
   .byte %00010000;|   X    |
   .byte %00010000;|   X    |
   .byte %00010000;|   X    |
   .byte %00010000;|   X    |
   
Level1
   .byte %01111100;| XXXXX  |
   .byte %01000000;| X      |
   .byte %01111001;| XXXX  X|
   .byte %01000001;| X     X|
   .byte %01111101;| XXXXX X|
   
Level2
   .byte %01000111;| X   XXX|
   .byte %10100100;|X X  X  |
   .byte %00010111;|   X XXX|
   .byte %00010100;|   X X  |
   .byte %00010111;|   X XXX|
   
Level3
   .byte %11011111;|XX XXXXX|
   .byte %00010000;|   X    |
   .byte %10010000;|X  X    |
   .byte %00010000;|   X    |
   .byte %11010000;|XX X    |

zero
   .byte $3C; |  XXXX  |
   .byte $66; | XX  XX |
   .byte $66; | XX  XX |
   .byte $66; | XX  XX |
   .byte $3C; |  XXXX  |
one
   .byte $3C; |  XXXX  |
   .byte $18; |   XX   |
   .byte $18; |   XX   |
   .byte $38; |  XXX   |
   .byte $18; |   XX   |
two
   .byte $7E; | XXXXXX |
   .byte $60; | XX     |
   .byte $3C; |  XXXX  |
   .byte $06; |     XX |
   .byte $7C; | XXXXX  |
three
   .byte $3C; |  XXXX  |
   .byte $46; | X   XX |
   .byte $0C; |    XX  |
   .byte $46; | X   XX |
   .byte $3C; |  XXXX  |
four   
   .byte $0C; |    XX  |
   .byte $0C; |    XX  |
   .byte $7E; | XXXXXX |   
   .byte $4C; | X  XX  |
   .byte $4C; | X  XX  |
five   
   .byte $7C; | XXXXX  |
   .byte $06; |     XX |
   .byte $7C; | XXXXX  |
   .byte $60; | XX     |
   .byte $7E; | XXXXXX |
six   
   .byte $3C; |  XXXX  |
   .byte $66; | XX  XX |
   .byte $7C; | XXXXX  |
   .byte $60; | XX     |
   .byte $3C; |  XXXX  |
seven   
   .byte $18; |   XX   |
   .byte $18; |   XX   |
   .byte $0C; |    XX  |
   .byte $06; |     XX |
   .byte $7E; | XXXXXX |
eight   
   .byte $3C; |  XXXX  |
   .byte $66; | XX  XX |
   .byte $3C; |  XXXX  |
   .byte $66; | XX  XX |
   .byte $3C; |  XXXX  |
nine   
   .byte $3C; |  XXXX  |
   .byte $06; |     XX |   
   .byte $3E; |  XXXXX |
   .byte $66; | XX  XX |
   .byte $3C; |  XXXX  |
   
Blank
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   .byte $00
   
   echo "***",*-$FB00, "BYTES OF ROM USED"
   .org $FFFC
   .word Start
   .word Start